@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -33,56 +33,35 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
-#include "gui/callbacks.h"
-#include "gui/progress_and_log.h"
-#include "algos/PSF.h"
+#include "core/siril_world_cs.h"
 #include "algos/photometry.h"
+#include "algos/sorting.h"
+#include "algos/siril_wcs.h"
+#include "filters/median.h"
+
+#include "PSF.h"
+
 
 #define MAX_ITER_NO_ANGLE  10		//Number of iteration in the minimization with no angle
 #define MAX_ITER_ANGLE     10		//Number of iteration in the minimization with angle
-#define EPSILON            0.01
+#define EPSILON            0.001
 
 const double radian_conversion = ((3600.0 * 180.0) / M_PI) / 1.0E3;
 
-static WORD getMedian3x3(gsl_matrix *in, const int xx, const int yy,
-		const int w, const int h) {
-	int step, radius, x, y;
-	double *value, median;
-
-	step = 1;
-	radius = 1;
-
-	int n = 0;
-	int start;
-	value = calloc(8, sizeof(double));
-	for (y = yy - radius; y <= yy + radius; y += step) {
-		for (x = xx - radius; x <= xx + radius; x += step) {
-			if (y >= 0 && y < h) {
-				if (x >= 0 && x < w) {
-					if ((x != xx) || (y != yy)) {
-						value[n++] = gsl_matrix_get(in, y, x);
-					}
-				}
-			}
-		}
-	}
-	start = 8 - n - 1;
-	quicksort_d(value, 8);
-	median = gsl_stats_median_from_sorted_data(value + start, 1, n);
-	free(value);
-	return median;
-}
-
 static gsl_matrix *removeHotPixels(gsl_matrix *in) {
-	int width = in->size2;
-	int height = in->size1;
-	int x, y;
+	size_t width = in->size2;
+	size_t height = in->size1;
+	size_t x, y;
 	gsl_matrix *out = gsl_matrix_alloc (in->size1, in->size2);
+	if (!out) {
+		PRINT_ALLOC_ERR;
+		return NULL;
+	}
 
 	gsl_matrix_memcpy (out, in);
 	for (y = 0; y < height; y++) {
 		for (x = 0; x < width; x++) {
-			double a = getMedian3x3(in, x, y, width, height);
+			double a = get_median_gsl(in, x, y, width, height, 1, FALSE, FALSE);
 			gsl_matrix_set(out, y, x, a);
 		}
 	}
@@ -100,6 +79,7 @@ static gsl_vector* psf_init_data(gsl_matrix* z, double bg) {
 	/* find maximum */
 	/* first we remove hot pixels in the matrix */
 	gsl_matrix *m_tmp = removeHotPixels(z);
+	if (!m_tmp) return NULL;
 	max = gsl_matrix_max(m_tmp);
 	gsl_matrix_max_index(m_tmp, &i, &j);
 	gsl_matrix_free(m_tmp);
@@ -312,20 +292,25 @@ static int psf_Gaussian_fdf_an(const gsl_vector * x, void *PSF_data,
 /* The function returns the fitted parameters without angle. However it
  * returns NULL if the number of parameters is => to the pixel number.
  */
-static fitted_PSF *psf_minimiz_no_angle(gsl_matrix* z, double background,
-		int layer) {
+static fitted_PSF *psf_minimiz_no_angle(gsl_matrix* z, double background) {
 	size_t i, j;
 	size_t NbRows = z->size1; //characteristics of the selection : height and width
 	size_t NbCols = z->size2;
 	const size_t p = 6;			// Number of parameters fitted
 	const size_t n = NbRows * NbCols;
-	fitted_PSF *psf = malloc(sizeof(fitted_PSF));
 	gsl_vector *MaxV = psf_init_data(z, background);
+	if (!MaxV)
+		return NULL;
 	int status;
 	unsigned int iter = 0;
 	gsl_matrix *covar = gsl_matrix_alloc(p, p);
 	double *y = malloc(n * sizeof(double));
 	double *sigma = malloc(n * sizeof(double));
+	fitted_PSF *psf = malloc(sizeof(fitted_PSF));
+	if (!y || !sigma || !psf) {
+		PRINT_ALLOC_ERR;
+		return NULL;
+	}
 	struct PSF_data d = { n, y, sigma, NbRows, NbCols, 0 };
 	gsl_multifit_function_fdf f;
 	const gsl_rng_type * type;
@@ -402,8 +387,7 @@ static fitted_PSF *psf_minimiz_no_angle(gsl_matrix* z, double background,
 	psf->units = "px";
 	// Magnitude
 	psf->mag = psf_get_mag(z, psf->B);
-	// Layer: not fitted
-	psf->layer = layer;
+	psf->phot_is_valid = FALSE;
 	// RMSE
 	psf->rmse = d.rmse;
 	// absolute uncertainties
@@ -430,19 +414,23 @@ static fitted_PSF *psf_minimiz_no_angle(gsl_matrix* z, double background,
  * NULL if the number of parameters is => to the pixel number.
  * This should not happen because this case is already treated by the
  * minimiz_no_angle function */
-static fitted_PSF *psf_minimiz_angle(gsl_matrix* z, fitted_PSF *psf, gboolean for_photometry) {
+static fitted_PSF *psf_minimiz_angle(gsl_matrix* z, fitted_PSF *psf, gboolean for_photometry, gboolean verbose) {
 	size_t i, j;
 	size_t NbRows = z->size1; //characteristics of the selection : height and width
 	size_t NbCols = z->size2;
 	const size_t p = 7;			// Number of parameters fitted
 	const size_t n = NbRows * NbCols;
 	g_assert (n > 0);
-	fitted_PSF *psf_angle = malloc(sizeof(fitted_PSF));
 	int status;
 	unsigned int iter = 0;
+	fitted_PSF *psf_angle = malloc(sizeof(fitted_PSF));
 	gsl_matrix *covar = gsl_matrix_alloc(p, p);
 	double *y = malloc(n * sizeof(double));
 	double *sigma = malloc(n * sizeof(double));
+	if (!psf_angle || !covar || !y || !sigma) {
+		PRINT_ALLOC_ERR;
+		return NULL;
+	}
 	struct PSF_data d = { n, y, sigma, NbRows, NbCols, 0 };
 	gsl_multifit_function_fdf f_angle;
 	double x_init[7] = { psf->B, psf->A, psf->x0, psf->y0, psf->sx, psf->sy, 0 };
@@ -467,7 +455,7 @@ static fitted_PSF *psf_minimiz_angle(gsl_matrix* z, fitted_PSF *psf, gboolean fo
 	for (i = 0; i < NbRows; i++) {
 		for (j = 0; j < NbCols; j++) {
 			y[NbCols * i + j] = gsl_matrix_get(z, i, j);
-			sigma[NbCols * i + j] = 1;
+			sigma[NbCols * i + j] = 1.0;
 		}
 	}
 
@@ -524,18 +512,22 @@ static fitted_PSF *psf_minimiz_angle(gsl_matrix* z, fitted_PSF *psf, gboolean fo
 	psf_angle->units = "px";
 	// Photometry
 	if (for_photometry)
-		psf_angle->phot = getPhotometryData(z, psf_angle);
-	else psf_angle->phot = NULL;
+		psf_angle->phot = getPhotometryData(z, psf_angle, verbose);
+	else {
+		psf_angle->phot = NULL;
+		psf_angle->phot_is_valid = FALSE;
+	}
 	// Magnitude
 	if (psf_angle->phot != NULL) {
 		psf_angle->mag = psf_angle->phot->mag;
 		psf_angle->s_mag = psf_angle->phot->s_mag;
+		psf_angle->phot_is_valid = psf_angle->phot->valid;
+
 	} else {
 		psf_angle->mag = psf_get_mag(z, psf_angle->B);
 		psf_angle->s_mag = 9.999;
+		psf_angle->phot_is_valid = FALSE;
 	}
-	//Layer: not fitted
-	psf_angle->layer = psf->layer;
 	//RMSE
 	psf_angle->rmse = d.rmse;
 	// absolute uncertainties
@@ -562,7 +554,7 @@ static fitted_PSF *psf_minimiz_angle(gsl_matrix* z, fitted_PSF *psf, gboolean fo
 /* Returns the largest FWHM.
  * The optional output parameter roundness is the ratio between the two axis FWHM */
 double psf_get_fwhm(fits *fit, int layer, double *roundness) {
-	fitted_PSF *result = psf_get_minimisation(fit, layer, &com.selection, FALSE);
+	fitted_PSF *result = psf_get_minimisation(fit, layer, &com.selection, FALSE, TRUE, TRUE);
 	if (result == NULL) {
 		*roundness = 0.0;
 		return 0.0;
@@ -579,29 +571,51 @@ double psf_get_fwhm(fits *fit, int layer, double *roundness) {
  * Selection rectangle is passed as third argument.
  * Return value is a structure, type fitted_PSF, that has to be freed after use.
  */
-fitted_PSF *psf_get_minimisation(fits *fit, int layer, rectangle *area, gboolean for_photometry) {
-	WORD *from;
+fitted_PSF *psf_get_minimisation(fits *fit, int layer, rectangle *area,
+		gboolean for_photometry, gboolean verbose, gboolean multithread_stat) {
 	int stridefrom, i, j;
 	fitted_PSF *result;
-	double bg = background(fit, layer, area);
-	//~ fprintf(stdout, "background: %g\n", bg);
+	double bg = background(fit, layer, area, multithread_stat);
 
-	// create the matrix with values from the selected rectangle
+	// fprintf(stdout, "background: %g\n", bg);
 	gsl_matrix *z = gsl_matrix_alloc(area->h, area->w);
-	from = fit->pdata[layer] + (fit->ry - area->y - area->h) * fit->rx
-			+ area->x;
 	stridefrom = fit->rx - area->w;
 
-	for (i = 0; i < area->h; i++) {
-		for (j = 0; j < area->w; j++) {
-			gsl_matrix_set(z, i, j, (double) *from);
-			from++;
+	// create the matrix with values from the selected rectangle
+	if (fit->type == DATA_USHORT) {
+		WORD *from = fit->pdata[layer] +
+			(fit->ry - area->y - area->h) * fit->rx + area->x;
+
+		for (i = 0; i < area->h; i++) {
+			for (j = 0; j < area->w; j++) {
+				gsl_matrix_set(z, i, j, (double)*from);
+				from++;
+			}
+			from += stridefrom;
 		}
-		from += stridefrom;
 	}
-	result = psf_global_minimisation(z, bg, layer, TRUE, for_photometry);
-	if (result)
-		psf_update_units(fit, &result);
+	else if (fit->type == DATA_FLOAT) {
+		float *from = fit->fpdata[layer] +
+			(fit->ry - area->y - area->h) * fit->rx + area->x;
+
+		for (i = 0; i < area->h; i++) {
+			for (j = 0; j < area->w; j++) {
+				gsl_matrix_set(z, i, j, (double)*from);
+				from++;
+			}
+			from += stridefrom;
+		}
+	}
+	else {
+		gsl_matrix_free(z);
+		return NULL;
+	}
+
+	result = psf_global_minimisation(z, bg, TRUE, for_photometry, verbose);
+	if (result) {
+		fwhm_to_arcsec_if_needed(fit, result);
+		result->layer = layer;
+	}
 	gsl_matrix_free(z);
 	return result;
 }
@@ -616,25 +630,45 @@ fitted_PSF *psf_get_minimisation(fits *fit, int layer, rectangle *area, gboolean
  * of the star_finder algorithm), no angle parameter is fitted.
  * The function returns NULL if values look bizarre.
  */
-fitted_PSF *psf_global_minimisation(gsl_matrix* z, double bg, int layer,
-		gboolean fit_angle, gboolean for_photometry) {
+fitted_PSF *psf_global_minimisation(gsl_matrix* z, double bg,
+		gboolean fit_angle, gboolean for_photometry, gboolean verbose) {
 	fitted_PSF *psf;
 
 	// To compute good starting values, we first compute with no angle
-	if ((psf = psf_minimiz_no_angle(z, bg, layer)) != NULL) {
-		if (fit_angle && fabs(psf->sx - psf->sy) >= EPSILON) {
-			fitted_PSF *tmp_psf;
-			if ((tmp_psf = psf_minimiz_angle(z, psf, for_photometry)) == NULL) {
+	if ((psf = psf_minimiz_no_angle(z, bg)) != NULL) {
+		if (fit_angle) {
+			/* This next check is to avoid possible angle divergence
+			 * when sx and sy are too close (star is quite round).
+			 * However, in this case we need to compute photometry if needed
+			 */
+			if ((fabs(psf->sx - psf->sy) < EPSILON)) {
+				// Photometry
+				if (for_photometry) {
+					psf->phot = getPhotometryData(z, psf, verbose);
+					if (psf->phot != NULL) {
+						psf->mag = psf->phot->mag;
+						psf->s_mag = psf->phot->s_mag;
+						psf->phot_is_valid = psf->phot->valid;
+					}
+				} else {
+					psf->phot = NULL;
+					psf->phot_is_valid = FALSE;
+				}
+			} else {
+				fitted_PSF *tmp_psf;
+				if ((tmp_psf = psf_minimiz_angle(z, psf, for_photometry, verbose))
+						== NULL) {
+					free(psf);
+					return NULL;
+				}
 				free(psf);
-				return NULL;
+				psf = tmp_psf;
 			}
-			free(psf);
-			psf = tmp_psf;
 		}
 		// Solve symmetry problem in order to have Sx>Sy in any case !!!
 		if (psf->sy > psf->sx) {
-			swap_param(&psf->sx, &psf->sy);
-			swap_param(&psf->fwhmx, &psf->fwhmy);
+			SWAP(psf->sx, psf->sy);
+			SWAP(psf->fwhmx, psf->fwhmy);
 			if (fit_angle && psf->angle != 0.0) {
 				if (psf->angle > 0.0)
 					psf->angle -= 90.0;
@@ -642,12 +676,6 @@ fitted_PSF *psf_global_minimisation(gsl_matrix* z, double bg, int layer,
 			}
 		}
 
-		/* We normalize B, A and the RMSE for the output */
-		WORD norm = get_normalized_value(&gfit);
-		psf->B = psf->B / (double) norm;
-		psf->A = psf->A / (double) norm;
-		psf->rmse = psf->rmse / (double) norm;
-
 		/* We quickly test the result. If it is bad we return NULL */
 		if (!isfinite(psf->fwhmx) || !isfinite(psf->fwhmy) ||
 				psf->fwhmx <= 0.0 || psf->fwhmy <= 0.0) {
@@ -661,39 +689,98 @@ fitted_PSF *psf_global_minimisation(gsl_matrix* z, double bg, int layer,
 }
 
 void psf_display_result(fitted_PSF *result, rectangle *area) {
-	char buffer[256];
+	char *buffer, *coordinates;
 	char *str;
 	if (com.magOffset > 0.0)
 		str = "true reduced";
 	else
 		str = "relative";
 
-	g_snprintf(buffer, sizeof(buffer), _("PSF fit Result:\n"
-			"x0=%0.2f px, y0=%0.2f px\n"
+	double x = result->x0 + area->x;
+	double y = area->y + area->h - result->y0;
+
+	if (has_wcs(&gfit)) {
+		double world_x, world_y;
+		SirilWorldCS *world_cs;
+		pix2wcs(&gfit, x, (double) gfit.ry - y, &world_x, &world_y);
+		world_cs = siril_world_cs_new_from_a_d(world_x, world_y);
+		if (world_cs) {
+			gchar *ra = siril_world_cs_alpha_format(world_cs, "%02dh%02dm%02ds");
+			gchar *dec = siril_world_cs_delta_format(world_cs, "%c%02d°%02d\'%02d\"");
+			coordinates = g_strdup_printf("x0=%0.2f px, y0=%0.2f px (%s , %s)", x, y, ra, dec);
+
+			siril_world_cs_unref(world_cs);
+			g_free(ra);
+			g_free(dec);
+		} else {
+			coordinates = g_strdup_printf("x0=%0.2f px, y0=%0.2f px", x, y);
+		}
+	} else {
+		coordinates = g_strdup_printf("x0=%0.2f px, y0=%0.2f px", x, y);
+	}
+
+	buffer = g_strdup_printf(_("PSF fit Result:\n"
+			"%s\n"
 			"FWHM X=%0.2f%s, FWHM Y=%0.2f%s\n"
 			"Angle=%0.2f deg\n"
 			"Background value=%0.6f\n"
 			"Maximal intensity=%0.6f\n"
 			"Magnitude (%s)=%0.2f\n"
-			"RMSE=%.3e\n"), result->x0 + area->x, area->y + area->h - result->y0,
+			"RMSE=%.3e\n"), coordinates,
 			result->fwhmx, result->units, result->fwhmy, result->units,
 			result->angle, result->B, result->A, str, result->mag + com.magOffset, result->rmse);
 
 	siril_log_message(buffer);
+	g_free(buffer);
+	g_free(coordinates);
 }
+
+#define _2_SQRT_2_LOG2 1.55185049709
+
 /* If the pixel pitch and the focal length are known and filled in the 
  * setting box, we convert FWHM in pixel to arcsec by multiplying
  * the FWHM value with the sampling value */
-void psf_update_units(fits* fit, fitted_PSF **result) {
+void fwhm_to_arcsec_if_needed(fits* fit, fitted_PSF *result) {
 
-	if (fit->focal_length <= 0.0 || fit->pixel_size_x <= 0.0
-			|| fit->pixel_size_y <= 0.0 || fit->binning_x <= 0
+	if (!result) return;
+	if (fit->focal_length <= 0.0 || fit->pixel_size_x <= 0.f
+			|| fit->pixel_size_y <= 0.f || fit->binning_x <= 0
 			|| fit->binning_y <= 0)
 		return;
 
-	(*result)->fwhmx *= (radian_conversion * fit->pixel_size_x
-			/ fit->focal_length) * (double) fit->binning_x;
-	(*result)->fwhmy *= (radian_conversion * fit->pixel_size_y
-			/ fit->focal_length) * (double) fit->binning_y;
-	(*result)->units = "\"";
+	double bin_X, bin_Y;
+	double fwhmx, fwhmy;
+
+	fwhmx = sqrt(result->sx * 0.5) * _2_SQRT_2_LOG2;
+	fwhmy = sqrt(result->sy * 0.5) * _2_SQRT_2_LOG2;
+
+	bin_X = fit->unbinned ? (double) fit->binning_x : 1.0;
+	bin_Y = fit->unbinned ? (double) fit->binning_y : 1.0;
+
+	result->fwhmx = fwhmx * (radian_conversion * (double)fit->pixel_size_x / fit->focal_length) * bin_X;
+	result->fwhmy = fwhmy * (radian_conversion * (double)fit->pixel_size_y / fit->focal_length) * bin_Y;
+	result->units = "\"";
 }
+
+void fwhm_to_pixels(fitted_PSF *result) {
+	result->fwhmx = sqrt(result->sx * 0.5) * _2_SQRT_2_LOG2;
+	result->fwhmy = sqrt(result->sy * 0.5) * _2_SQRT_2_LOG2;
+	result->units = "px";
+}
+
+double convert_single_fwhm_to_pixels(double fwhm, double s) {
+	return sqrt(s * 0.5) * _2_SQRT_2_LOG2;
+}
+
+fitted_PSF *duplicate_psf(fitted_PSF *psf) {
+	if (!psf)
+		return NULL;
+	fitted_PSF *new_psf = malloc(sizeof(fitted_PSF));
+	memcpy(new_psf, psf, sizeof(fitted_PSF));
+	if (psf->phot) {
+		new_psf->phot = malloc(sizeof(photometry));
+		memcpy(new_psf->phot, psf->phot, sizeof(photometry));
+	}
+	return new_psf;
+}
+
