@@ -16,8 +16,10 @@ struct fwhm_struct {
 	double mag; /* magnitude of the star : this parameter is not fitted but calculated with the vector G and the parameter B */
 	double s_mag; /* error on the magnitude */
 	photometry *phot; /* photometry data */
+	gboolean phot_is_valid; /* valid if computed by photometry and no saturated pixel detected */
 	double xpos, ypos; /* position of the star in the image, not set by Minimization */
 	double rmse; /* RMSE of the minimization */
+	double BV; /* only use in BV calibration */
 
 	/* uncertainties */
 	double B_err;
@@ -39,9 +41,13 @@ struct PSF_data {
 };
 
 double psf_get_fwhm(fits *, int, double *);
-fitted_PSF *psf_get_minimisation(fits *, int, rectangle *, gboolean);
-fitted_PSF *psf_global_minimisation(gsl_matrix *, double, int, gboolean, gboolean);
+fitted_PSF *psf_get_minimisation(fits *, int, rectangle *, gboolean, gboolean, gboolean);
+fitted_PSF *psf_global_minimisation(gsl_matrix *, double, gboolean, gboolean, gboolean);
 void psf_display_result(fitted_PSF *, rectangle *);
-void psf_update_units(fits*, fitted_PSF**);
+void fwhm_to_arcsec_if_needed(fits*, fitted_PSF*);
+void fwhm_to_pixels(fitted_PSF *result);
+double convert_single_fwhm_to_pixels(double fwhm, double s);
+
+fitted_PSF *duplicate_psf(fitted_PSF *);
 
 #endif
