@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -24,8 +24,6 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
-#include "gui/callbacks.h"
-#include "gui/progress_and_log.h"
 #include "algos/PSF.h"
 #include "algos/photometry.h"
 
@@ -36,14 +34,6 @@
 #define epsilon(x) 0.00000001
 #define maxit      50
 #define min_sky    5
-#define lo_data    0.0
-#define hi_data    USHRT_MAX_DOUBLE
-
-static void initializeParam() {
-	com.phot_set.inner = 20;
-	com.phot_set.outer = 30;
-	com.phot_set.gain = 2.3;
-}
 
 static double hampel(double x) {
 	if (x >= 0) {
@@ -84,7 +74,7 @@ static double dhampel(double x) {
 }
 
 static double qmedD(int n, double *a)
-/* Vypocet medianu algoritmem Quick Median (Wirth) */
+	/* Vypocet medianu algoritmem Quick Median (Wirth) */
 {
 	double w, x;
 	int i, j;
@@ -118,7 +108,7 @@ static double qmedD(int n, double *a)
 }
 
 static int robustmean(int n, double *x, double *mean, double *stdev)
-/* Newton's iterations */
+	/* Newton's iterations */
 {
 	int i, it;
 	double a, c, d, dt, r, s, sum1, sum2, sum3, psir;
@@ -140,9 +130,13 @@ static int robustmean(int n, double *x, double *mean, double *stdev)
 	}
 
 	/* initial values:
-	 - median is the first approximation of location
-	 - MAD/0.6745 is the first approximation of scale */
+	   - median is the first approximation of location
+	   - MAD/0.6745 is the first approximation of scale */
 	xx = malloc(n * sizeof(double));
+	if (!xx) {
+		PRINT_ALLOC_ERR;
+		return 1;
+	}
 	memcpy(xx, x, n * sizeof(double));
 	a = qmedD(n, xx);
 	for (i = 0; i < n; i++)
@@ -190,14 +184,14 @@ static int robustmean(int n, double *x, double *mean, double *stdev)
 	return 0;
 }
 
-static photometry *phot_alloc(size_t size) {
+static photometry *phot_alloc() {
 	photometry *phot;
 
-	phot = (photometry*) calloc((unsigned) size * sizeof(photometry), 1);
+	phot = (photometry*) calloc(sizeof(photometry), 1);
 	if (phot == NULL) {
-		printf("photometry: memory error\n");
+		PRINT_ALLOC_ERR;
 	}
-	return (phot);
+	return phot;
 }
 
 static double getMagnitude(double intensity) {
@@ -205,15 +199,19 @@ static double getMagnitude(double intensity) {
 }
 
 static double getCameraGain() {
-	return com.phot_set.gain;
+	if (gfit.type == DATA_FLOAT) {
+		return com.pref.phot_set.gain * USHRT_MAX_DOUBLE;
+	} else {
+		return com.pref.phot_set.gain;
+	}
 }
 
 static double getInnerRadius() {
-	return com.phot_set.inner;
+	return com.pref.phot_set.inner;
 }
 
 static double getOuterRadius() {
-	return com.phot_set.outer;
+	return com.pref.phot_set.outer;
 }
 
 static double getMagErr(double intensity, double area, int nsky, double skysig) {
@@ -231,8 +229,24 @@ static double getMagErr(double intensity, double area, int nsky, double skysig)
 	return fmin(9.999, 1.0857 * sqrt(err1 + err2 + err3) / intensity);
 }
 
+static double lo_data() {
+	if (gfit.type == DATA_FLOAT) {
+		return (double) com.pref.phot_set.minval / USHRT_MAX_DOUBLE;
+	} else {
+		return (double) com.pref.phot_set.minval;
+	}
+}
+
+static double hi_data() {
+	if (gfit.type == DATA_FLOAT) {
+		return (double) com.pref.phot_set.maxval / USHRT_MAX_DOUBLE;
+	} else {
+		return (double) com.pref.phot_set.maxval;
+	}
+}
+
 /* Function that compute all photometric data. The result must be freed */
-photometry *getPhotometryData(gsl_matrix* z, fitted_PSF *psf) {
+photometry *getPhotometryData(gsl_matrix* z, fitted_PSF *psf, gboolean verbose) {
 	int width = z->size2;
 	int height = z->size1;
 	int n_sky = 0, ret;
@@ -241,18 +255,26 @@ photometry *getPhotometryData(gsl_matrix* z, fitted_PSF *psf) {
 	double xc, yc;
 	double apmag = 0.0, mean = 0.0, stdev = 0.0, area = 0.0;
 	double signalIntensity;
-	double *data;
+	gboolean valid = TRUE;
 	photometry *phot;
 
 	xc = psf->x0 - 1;
 	yc = psf->y0 - 1;
 
+	if ((xc <= 0.0) || (yc <= 0.0)) return NULL;
+
 	r1 = getInnerRadius();
 	r2 = getOuterRadius();
-	appRadius = sqrt(psf->sx / 2.0) * 2 * sqrt(log(2.0) * 2) + 0.5;
+	appRadius = psf->fwhmx;
+	if (psf->units[0] == '\"') {
+		appRadius = convert_single_fwhm_to_pixels(psf->fwhmx, psf->sx);
+	}
+	appRadius *= 2; // in order to be sure to contain star
 	if (appRadius >= r1) {
-		/* Translator note: radii is plural for radius */
-		siril_log_message(_("Inner and outer radii are too small. Please update values in setting box.\n"));
+		if (verbose) {
+			/* Translator note: radii is plural for radius */
+			siril_log_message(_("Inner and outer radii are too small. Please update values in preferences.\n"));
+		}
 		return NULL;
 	}
 
@@ -273,14 +295,23 @@ photometry *getPhotometryData(gsl_matrix* z, fitted_PSF *psf) {
 	r2 *= r2;
 	rmin_sq = (appRadius - 0.5) * (appRadius - 0.5);
 
-	data = calloc((y2 - y1) * (x2 - x1), sizeof(double));
+	int ndata = (y2 - y1) * (x2 - x1);
+	if (ndata <= 0) {
+		siril_log_color_message(_("An error occurred in your selection. Please make another selection.\n"), "red");
+		return NULL;
+	}
+	double *data = calloc(ndata, sizeof(double));
+	if (!data) {
+		PRINT_ALLOC_ERR;
+		return NULL;
+	}
 
 	for (y = y1; y <= y2; ++y) {
 		int yp = (y - yc) * (y - yc);
 		for (x = x1; x <= x2; ++x) {
 			r = yp + (x - xc) * (x - xc);
 			double pixel = gsl_matrix_get(z, y, x);
-			if (pixel > lo_data && pixel < hi_data) {
+			if (pixel > lo_data() && pixel < hi_data()) {
 				double f = (r < rmin_sq ? 1 : appRadius - sqrt(r) + 0.5);
 				if (f >= 0) {
 					area += f;
@@ -291,6 +322,8 @@ photometry *getPhotometryData(gsl_matrix* z, fitted_PSF *psf) {
 					data[n_sky] = pixel;
 					n_sky++;
 				}
+			} else {
+				valid = FALSE;
 			}
 		}
 	}
@@ -298,38 +331,37 @@ photometry *getPhotometryData(gsl_matrix* z, fitted_PSF *psf) {
 		free(data);
 		return NULL;
 	}
-
 	if (n_sky < min_sky) {
-		siril_log_message(_("Warning: There aren't enough pixels"
-				" in the sky annulus. You need to make a larger selection.\n"));
+		if (verbose) {
+			siril_log_message(_("Warning: There aren't enough pixels"
+						" in the sky annulus. You need to make a larger selection.\n"));
+		}
 		free(data);
 		return NULL;
 	}
 
 	ret = robustmean(n_sky, data, &mean, &stdev);
+	free(data);
 	if (ret > 0) {
-		free(data);
 		return NULL;
 	}
 
-	phot = phot_alloc(1);
+	phot = phot_alloc();
 	if (phot) {
 		signalIntensity = apmag - (area * mean);
+
 		phot->mag = getMagnitude(signalIntensity);
 		phot->s_mag = getMagErr(signalIntensity, area, n_sky, stdev);
+		phot->valid = valid;
 	}
 
-	free(data);
 	return phot;
 }
 
-void on_button_reset_photometry_clicked(GtkButton *button, gpointer user_data) {
-	double tmp;
-
-	initializeParam();
-	tmp = gfit.cvf;
-	gfit.cvf = 0.0;
-	set_GUI_photometry();
-	gfit.cvf = tmp;
+void initialize_photometric_param() {
+	com.pref.phot_set.inner = 20;
+	com.pref.phot_set.outer = 30;
+	com.pref.phot_set.gain = 2.3;
+	com.pref.phot_set.minval = 0;
+	com.pref.phot_set.maxval = 60000;
 }
-
