@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -29,28 +29,35 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "algos/Def_Wavelet.h"
-#include "gui/callbacks.h"
+#include "gui/utils.h"
+#include "gui/message_dialog.h"
+#include "gui/image_display.h"
 #include "gui/progress_and_log.h"
-#include "algos/PSF.h"
 #include "gui/PSF_list.h"
+#include "algos/PSF.h"
 #include "algos/star_finder.h"
 #include "algos/statistics.h"
+#include "filters/wavelets.h"
+#include "io/image_format_fits.h"
+#include "core/OS_utils.h"
 
 #define WAVELET_SCALE 3
 
-static WORD Compute_threshold(fits *fit, double starfinder, int layer, WORD *norm, double *bg) {
-	WORD threshold;
+static float compute_threshold(fits *fit, double ksigma, int layer, float *norm, double *bg) {
+	float threshold;
 	imstats *stat;
 
 	assert(layer <= 3);
 
-	stat = statistics(NULL, -1, fit, layer, NULL, STATS_BASIC);
+	stat = statistics(NULL, -1, fit, layer, NULL, STATS_BASIC, FALSE);
 	if (!stat) {
 		siril_log_message(_("Error: statistics computation failed.\n"));
+		*norm = 0;
+		*bg = 0.0;
 		return 0;
 	}
-	threshold = (WORD) stat->median + starfinder * (WORD) stat->sigma;
-	*norm = (WORD) stat->normValue;
+	threshold = (float)(stat->median + ksigma * stat->sigma);
+	*norm = stat->normValue;
 	*bg = stat->median;
 	free_stats(stat);
 
@@ -66,15 +73,12 @@ static gboolean is_star(fitted_PSF *result, star_finder_params *sf) {
 		return FALSE;
 	if ((result->x0 <= 0.0) || (result->y0 <= 0.0))
 		return FALSE;
-	if (result->A < 0.01)
-		return FALSE;
 	if (result->sx > 200 || result->sy > 200)
 		return FALSE;
 	if (result->fwhmx <= 0.0 || result->fwhmy <= 0.0)
 		return FALSE;
 	if ((result->fwhmy / result->fwhmx) < sf->roundness)
 		return FALSE;
-
 	return TRUE;
 }
 
@@ -102,9 +106,9 @@ void init_peaker_GUI() {
 
 void init_peaker_default() {
 	/* values taken from siril3.glade */
-	com.starfinder_conf.radius = 7;
+	com.starfinder_conf.radius = 10;
 	com.starfinder_conf.sigma = 1.0;
-	com.starfinder_conf.roundness = 0.6;
+	com.starfinder_conf.roundness = 0.5;
 }
 
 void on_spin_sf_radius_changed(GtkSpinButton *spinbutton, gpointer user_data) {
@@ -119,6 +123,20 @@ void on_spin_sf_roundness_changed(GtkSpinButton *spinbutton, gpointer user_data)
 	com.starfinder_conf.roundness = gtk_spin_button_get_value(spinbutton);
 }
 
+void update_peaker_GUI() {
+	static GtkSpinButton *spin_radius = NULL, *spin_sigma = NULL,
+			*spin_roundness = NULL;
+
+	if (spin_radius == NULL) {
+		spin_radius = GTK_SPIN_BUTTON(lookup_widget("spinstarfinder_radius"));
+		spin_sigma = GTK_SPIN_BUTTON(lookup_widget("spinstarfinder_threshold"));
+		spin_roundness = GTK_SPIN_BUTTON(lookup_widget("spinstarfinder_round"));
+	}
+	gtk_spin_button_set_value(spin_radius, (double) com.starfinder_conf.radius);
+	gtk_spin_button_set_value(spin_sigma, com.starfinder_conf.sigma);
+	gtk_spin_button_set_value(spin_roundness, com.starfinder_conf.roundness);
+}
+
 /*
  This is an implementation of a simple peak detector algorithm which
  identifies any pixel that is greater than any of its eight neighbors.
@@ -126,8 +144,8 @@ void on_spin_sf_roundness_changed(GtkSpinButton *spinbutton, gpointer user_data)
  Original algorithm come from:
  Copyleft (L) 1998 Kenneth J. Mighell (Kitt Peak National Observatory)
  */
+static int minimize_candidates(fits *image, star_finder_params *sf, double bg, pointi *candidates, int nb_candidates, int layer, fitted_PSF ***retval);
 
-/* returns a NULL-ended array of FWHM info */
 fitted_PSF **peaker(fits *fit, int layer, star_finder_params *sf, int *nb_stars, rectangle *area, gboolean showtime) {
 	int nx = fit->rx;
 	int ny = fit->ry;
@@ -135,53 +153,49 @@ fitted_PSF **peaker(fits *fit, int layer, star_finder_params *sf, int *nb_stars,
 	int areaY0 = 0;
 	int areaX1 = nx;
 	int areaY1 = ny;
-	int y, k, nbstars = 0;
+	int nbstars = 0;
 	double bg;
-	WORD threshold, norm;
-	WORD **wave_image, **real_image;
+	float threshold, norm;
+	float **wave_image;
 	fits wave_fit = { 0 };
-	fitted_PSF **results;
+	pointi *candidates;
+
 	struct timeval t_start, t_end;
 
 	assert(nx > 0 && ny > 0);
 
-	results = malloc((MAX_STARS + 1) * sizeof(fitted_PSF *));
-	if (results == NULL) {
-		printf("Memory allocation failed: peaker\n");
-		return NULL;
-	}
-
-	siril_log_color_message(_("Findstar: processing...\n"), "red");
+	siril_log_color_message(_("Findstar: processing...\n"), "green");
 	gettimeofday(&t_start, NULL);
 
-	results[0] = NULL;
-	threshold = Compute_threshold(fit, sf->sigma, layer, &norm, &bg);
+	/* running statistics on the input image is best as it caches them */
+	threshold = compute_threshold(fit, sf->sigma, layer, &norm, &bg);
+	if (norm == 0.0f)
+		return NULL;
 
-	copyfits(fit, &wave_fit, CP_ALLOC | CP_FORMAT | CP_COPYA, 0);
-	get_wavelet_layers(&wave_fit, WAVELET_SCALE, 2, TO_PAVE_BSPLINE, layer);
+	siril_debug_print("Threshold: %f (background: %f, norm: %f)\n", threshold, bg, norm);
 
-	/* FILL wavelet image upside-down */
-	wave_image = malloc(ny * sizeof(WORD *));
-	if (wave_image == NULL) {
-		free(results);
-		clearfits(&wave_fit);
-		fprintf(stderr, "Memory allocation failed: peaker\n");
+	/* if fit is ushort, we need to get it in float [0, 65535] to run
+	 * statistics only once: stats give the threshold used in the
+	 * wavelets-filtered image for candidate detection and the background
+	 * used in PSF fitting from real image data which can be still ushort.
+	 */
+	if (extract_fits(fit, &wave_fit, layer, TRUE)) {
+		siril_log_message(_("Failed to copy the image for processing\n"));
 		return NULL;
 	}
-	for (k = 0; k < ny; k++)
-		wave_image[ny - k - 1] = wave_fit.pdata[layer] + k * nx;
 
-	/* FILL real image upside-down */
-	real_image = malloc(ny * sizeof(WORD *));
-	if (real_image == NULL) {
-		free(results);
-		free(wave_image);
+	get_wavelet_layers(&wave_fit, WAVELET_SCALE, 2, TO_PAVE_BSPLINE, 0);
+
+	/* Build 2D representation of wavelet image upside-down */
+	wave_image = malloc(ny * sizeof(float *));
+	if (!wave_image) {
+		PRINT_ALLOC_ERR;
 		clearfits(&wave_fit);
-		fprintf(stderr, "Memory allocation failed: peaker\n");
 		return NULL;
 	}
-	for (k = 0; k < ny; k++)
-		real_image[ny - k - 1] = fit->pdata[layer] + k * nx;
+	for (int k = 0; k < ny; k++) {
+		wave_image[ny - k - 1] = wave_fit.fdata + k * nx;
+	}
 
 	if (area) {
 		areaX0 = area->x;
@@ -190,16 +204,21 @@ fitted_PSF **peaker(fits *fit, int layer, star_finder_params *sf, int *nb_stars,
 		areaY1 = area->h + areaY0;
 	}
 
-	for (y = sf->radius + areaY0; y < areaY1 - sf->radius; y++) {
-		int x;
-		for (x = sf->radius + areaX0; x < areaX1 - sf->radius; x++) {
-			WORD pixel = wave_image[y][x];
+	candidates = malloc(MAX_STARS * sizeof(pointi));
+	if (!candidates) {
+		PRINT_ALLOC_ERR;
+		return NULL;
+	}
+
+	/* Search for candidate stars in the filtered image */
+	for (int y = sf->radius + areaY0; y < areaY1 - sf->radius; y++) {
+		for (int x = sf->radius + areaX0; x < areaX1 - sf->radius; x++) {
+			float pixel = wave_image[y][x];
 			if (pixel > threshold && pixel < norm) {
-				int yy, xx;
 				gboolean bingo = TRUE;
-				WORD neighbor;
-				for (yy = y - 1; yy <= y + 1; yy++) {
-					for (xx = x - 1; xx <= x + 1; xx++) {
+				float neighbor;
+				for (int yy = y - 1; yy <= y + 1; yy++) {
+					for (int xx = x - 1; xx <= x + 1; xx++) {
 						if (xx == x && yy == y)
 							continue;
 						neighbor = wave_image[yy][xx];
@@ -215,50 +234,25 @@ fitted_PSF **peaker(fits *fit, int layer, star_finder_params *sf, int *nb_stars,
 					}
 				}
 				if (bingo && nbstars < MAX_STARS) {
-					int ii, jj, i, j;
-					//~ fprintf(stdout, "Found a probable star at position (%d, %d) with a value of %hu\n", x, y, pixel);
-					gsl_matrix *z = gsl_matrix_alloc(sf->radius * 2, sf->radius * 2);
-					/* FILL z */
-					for (jj = 0, j = y - sf->radius; j < y + sf->radius;
-							j++, jj++) {
-						for (ii = 0, i = x - sf->radius; i < x + sf->radius;
-								i++, ii++) {
-							gsl_matrix_set(z, ii, jj, (double) real_image[j][i]);
-						}
-					}
-					/* ****** */
-					/* In this case the angle is not fitted because it
-					 *  slows down the algorithm too much 
-					 * To fit the angle, set the 4th parameter to TRUE */
-					fitted_PSF *cur_star = psf_global_minimisation(z, bg, layer,
-							FALSE, FALSE);
-					if (cur_star) {
-						psf_update_units(fit, &cur_star);
-						if (is_star(cur_star, sf)) {
-							cur_star->xpos = x + cur_star->x0 - sf->radius - 1;
-							cur_star->ypos = y + cur_star->y0 - sf->radius - 1;
-							if (nbstars < MAX_STARS) {
-								results[nbstars] = cur_star;
-								results[nbstars + 1] = NULL;
-								nbstars++;
-							}
-
-						}
-					}
-					gsl_matrix_free(z);
+					candidates[nbstars].x = x;
+					candidates[nbstars].y = y;
+					nbstars++;
+					if (nbstars == MAX_STARS) break;
 				}
 			}
 		}
+		if (nbstars == MAX_STARS) break;
 	}
+	clearfits(&wave_fit);
+	siril_debug_print("Candidates for stars: %d\n", nbstars);
 
-	if (nbstars == 0) {
-		free(results);
+	/* Check if candidates are stars by minimizing a PSF on each */
+	fitted_PSF **results;
+	nbstars = minimize_candidates(fit, sf, bg, candidates, nbstars, layer, &results);
+	if (nbstars == 0)
 		results = NULL;
-	}
 	sort_stars(results, nbstars);
 	free(wave_image);
-	free(real_image);
-	clearfits(&wave_fit);
 
 	gettimeofday(&t_end, NULL);
 	if (showtime)
@@ -268,6 +262,79 @@ fitted_PSF **peaker(fits *fit, int layer, star_finder_params *sf, int *nb_stars,
 	return results;
 }
 
+/* returns number of stars found, result is in parameters */
+static int minimize_candidates(fits *image, star_finder_params *sf, double bg, pointi *candidates, int nb_candidates, int layer, fitted_PSF ***retval) {
+	int radius = sf->radius;
+	int nx = image->rx;
+	int ny = image->ry;
+	gsl_matrix *z = gsl_matrix_alloc(radius * 2, radius * 2);
+	WORD **image_ushort = NULL;
+	float **image_float = NULL;
+	gint nbstars = 0;
+
+	if (image->type == DATA_USHORT) {
+		image_ushort = malloc(ny * sizeof(WORD *));
+		for (int k = 0; k < ny; k++)
+			image_ushort[ny - k - 1] = image->pdata[layer] + k * nx;
+	}
+	else if (image->type == DATA_FLOAT) {
+		image_float = malloc(ny * sizeof(float *));
+		for (int k = 0; k < ny; k++)
+			image_float[ny - k - 1] = image->fpdata[layer] + k * nx;
+	}
+	else return 0;
+
+	fitted_PSF **results = malloc((nb_candidates + 1) * sizeof(fitted_PSF *));
+	if (!results) {
+		PRINT_ALLOC_ERR;
+		return 0;
+	}
+
+	for (int candidate = 0; candidate < nb_candidates; candidate++) {
+		int x = candidates[candidate].x, y = candidates[candidate].y;
+		int ii, jj, i, j;
+		/* FILL z */
+		if (image->type == DATA_USHORT) {
+			for (jj = 0, j = y - radius; j < y + radius; j++, jj++) {
+				for (ii = 0, i = x - radius; i < x + radius;
+						i++, ii++) {
+					gsl_matrix_set(z, ii, jj, (double)image_ushort[j][i]);
+				}
+			}
+		} else {
+			for (jj = 0, j = y - radius; j < y + radius; j++, jj++) {
+				for (ii = 0, i = x - radius; i < x + radius;
+						i++, ii++) {
+					gsl_matrix_set(z, ii, jj, (double)image_float[j][i]);
+				}
+			}
+		}
+
+		fitted_PSF *cur_star = psf_global_minimisation(z, bg, FALSE, FALSE, FALSE);
+		if (cur_star) {
+			if (is_star(cur_star, sf)) {
+				//fwhm_to_arcsec_if_needed(image, cur_star);	// should we do this here?
+				cur_star->layer = layer;
+				int result_index = g_atomic_int_add(&nbstars, 1);
+				cur_star->xpos = (x - radius) + cur_star->x0 - 1.0;
+				cur_star->ypos = (y - radius) + cur_star->y0 - 1.0;
+				results[result_index] = cur_star;
+				//fprintf(stdout, "%03d: %11f %11f %f\n",
+				//		result_index, cur_star->xpos, cur_star->ypos, cur_star->mag);
+			}
+			else free(cur_star);
+		}
+	}
+
+	results[nbstars] = NULL;
+	if (retval)
+		*retval = results;
+	gsl_matrix_free(z);
+	if (image_ushort) free(image_ushort);
+	if (image_float) free(image_float);
+	return nbstars;
+}
+
 /* Function to add star one by one, from the selection rectangle, the
  * minimization is run and the star is detected and added to the list of stars.
  *
@@ -282,7 +349,7 @@ fitted_PSF *add_star(fits *fit, int layer, int *index) {
 	gboolean already_found = FALSE;
 
 	*index = -1;
-	fitted_PSF * result = psf_get_minimisation(&gfit, layer, &com.selection, FALSE);
+	fitted_PSF * result = psf_get_minimisation(&gfit, layer, &com.selection, FALSE, TRUE, TRUE);
 	if (!result)
 		return NULL;
 	/* We do not check if it's matching with the "is_star()" criteria.
@@ -300,8 +367,10 @@ fitted_PSF *add_star(fits *fit, int layer, int *index) {
 		}
 	} else {
 		com.stars = malloc((MAX_STARS + 1) * sizeof(fitted_PSF*));
-		if (com.stars == NULL)
+		if (!com.stars) {
+			PRINT_ALLOC_ERR;
 			return NULL;
+		}
 		com.star_is_seqdata = FALSE;
 	}
 
@@ -309,7 +378,7 @@ fitted_PSF *add_star(fits *fit, int layer, int *index) {
 		free(result);
 		result = NULL;
 		char *msg = siril_log_message(_("This star has already been picked !\n"));
-		show_dialog(msg, _("Peaker"), "dialog-information-symbolic");
+		siril_message_dialog( GTK_MESSAGE_INFO, _("Peaker"), msg);
 	} else {
 		if (i < MAX_STARS) {
 			result->xpos = result->x0 + com.selection.x;
@@ -327,7 +396,6 @@ fitted_PSF *add_star(fits *fit, int layer, int *index) {
 
 int get_size_star_tab() {
 	int i = 0;
-
 	while (com.stars[i])
 		i++;
 	return i;
@@ -364,13 +432,21 @@ void sort_stars(fitted_PSF **stars, int total) {
 		qsort(*(&stars), total, sizeof(fitted_PSF*), compare_stars);
 }
 
-void FWHM_average(fitted_PSF **stars, float *FWHMx, float *FWHMy, int max) {
+void free_fitted_stars(fitted_PSF **stars) {
+	int i = 0;
+	while (stars && stars[i])
+		free(stars[i++]);
+	free(stars);
+}
+
+void FWHM_average(fitted_PSF **stars, int nb, float *FWHMx, float *FWHMy, char **units) {
 	if (stars && stars[0]) {
 		int i = 0;
 
 		*FWHMx = 0.0f;
 		*FWHMy = 0.0f;
-		while (i < max) {
+		*units = stars[0]->units;
+		while (i < nb) {
 			*FWHMx += (float) stars[i]->fwhmx;
 			*FWHMy += (float) stars[i]->fwhmy;
 			i++;
