@@ -1,6 +1,16 @@
 #ifndef _SIRIL_STATS_H
 #define _SIRIL_STATS_H
 
+struct stat_data {
+	fits *fit;
+	int option;
+	gchar **list;
+	sequence *seq;
+	gchar *csv_name;
+	const gchar *seqEntry;	// not used for stats
+
+};
+
 #define STATS_MINMAX	(1 << 1)	// min, max
 #define STATS_NOISE (1 << 2) // noise
 #define STATS_BASIC	(1 << 3)	// median, mean, sigma, noise, min, max
@@ -16,7 +26,9 @@
 #include "core/siril.h"
 
 imstats* statistics(sequence *seq, int image_index, fits *fit, int layer,
-		rectangle *selection, int option);
+		rectangle *selection, int option, gboolean multithread);
+
+int compute_means_from_flat_cfa(fits *fit, float mean[4]);
 
 void allocate_stats(imstats **stat);
 imstats* free_stats(imstats *stat);
@@ -31,5 +43,11 @@ void save_stats_from_fit(fits *fit, sequence *seq, int index);
 void invalidate_stats_from_fit(fits *fit);
 void full_stats_invalidation_from_fit(fits *fit);
 
+void apply_stats_to_sequence(struct stat_data *stat_args);
+
+float siril_stats_ushort_sd_64(const WORD data[], const int N);
+float siril_stats_ushort_sd_32(const WORD data[], const int N);
+float siril_stats_float_sd(const float data[], const int N);
+
 #endif
 
