@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -32,7 +32,7 @@ narrow_filter narrow_band_filters[] = {
 };
 
 int get_nb_narrow_filters() {
-	return sizeof(narrow_band_filters) / sizeof(narrow_filter);
+	return G_N_ELEMENTS(narrow_band_filters);
 }
 
 /*
