@@ -1,6 +1,9 @@
 #ifndef _PROCESSING_H_
 #define _PROCESSING_H_
 
+#include "sequence_filtering.h"
+#include "io/fits_sequence.h"
+
 /**
  *
  * \file processing.h
@@ -8,13 +11,12 @@
  *
  */
 
-/** the dynamic image selection, based on various possible criteria */
-typedef int (*seq_image_filter)(sequence *seq, int nb_img, double param);
-
 /** Main structure of the generic function */
 struct generic_seq_args {
 	/** sequence that will be processed */
 	sequence *seq;
+	/** read images as float data in all cases */
+	gboolean force_float;
 
 	/** process a partial image read from area instead of full-frame reading */
 	gboolean partial_image;
@@ -35,6 +37,8 @@ struct generic_seq_args {
 	 *  progress report. < 1 is unknown */
 	int nb_filtered_images;
 
+	/** function called to compute the required disk size if has_output */
+	gint64 (*compute_size_hook)(struct generic_seq_args *, int);
 	/** function called before iterating through the sequence */
 	int (*prepare_hook)(struct generic_seq_args *);
 	/** function called for each image with image index in sequence, number
@@ -64,6 +68,10 @@ struct generic_seq_args {
 
 	/** some processing may create a new image sequence */
 	gboolean has_output;
+	/** the type of the created sequence, for disk space checks only */
+	data_type output_type;
+	/** size ratio of output images for memory evaluation */
+	double upscale_ratio;
 	/** output files: prefix for the new sequence and automatic loading */
 	const char *new_seq_prefix;
 	/** flag to load or not a new sequence */
@@ -72,14 +80,22 @@ struct generic_seq_args {
 	gboolean force_ser_output;
 	/** new output SER if seq->type == SEQ_SER or force_ser_output (internal) */
 	struct ser_struct *new_ser;
+	/** flag to force output to be FITS sequence file */
+	gboolean force_fitseq_output;
+	/** new output SER if seq->type == SEQ_FITSEQ or force_fitseq_output (internal) */
+	fitseq *new_fitseq;
 
 	/** user data: pointer to operation-specific data */
 	void *user;
 
-	/** do not run the sequence processing in a new thread */
+	/** if the generic sequence processing is run from an existing thread,
+	 * the idle function is not executed in the GTK+ main thread but in this
+	 * same thread. If this is false, the generic idle function is run. */
 	gboolean already_in_a_thread;
 	/** activate parallel execution */
 	gboolean parallel;
+	/** number of threads to run in parallel - defaults to com.max_thread */
+	int max_thread;
 #ifdef _OPENMP
 	/** for in-hook synchronization (internal init, public use) */
 	omp_lock_t lock;
@@ -89,20 +105,24 @@ struct generic_seq_args {
 gpointer generic_sequence_worker(gpointer p);
 gboolean end_generic_sequence(gpointer p);
 
-int ser_prepare_hook(struct generic_seq_args *args);
-int ser_finalize_hook(struct generic_seq_args *args);
+int seq_prepare_hook(struct generic_seq_args *args);
+int seq_prepare_writer(struct generic_seq_args *args);
+int seq_finalize_hook(struct generic_seq_args *args);
 int generic_save(struct generic_seq_args *, int, int, fits *);
 
-int seq_filter_all(sequence *seq, int nb_img, double any);
-int seq_filter_included(sequence *seq, int nb_img, double any);
-
 void start_in_new_thread(gpointer(*f)(gpointer p), gpointer p);
 gpointer waiting_for_thread();
 void stop_processing_thread();
 void set_thread_run(gboolean b);
 gboolean get_thread_run();
 
+void start_in_reserved_thread(gpointer (*f)(gpointer), gpointer p);
+gboolean reserve_thread();
+void unreserve_thread();
+
 gboolean end_generic(gpointer arg);
 guint siril_add_idle(GSourceFunc idle_function, gpointer data);
 
+struct generic_seq_args *create_default_seqargs();
+
 #endif
