@@ -6,6 +6,7 @@
 
 #include <glib.h>
 #include <glib/gstdio.h>
+#include <glib/gprintf.h>
 #include <gtk/gtk.h>
 #include <fitsio.h>	// fitsfile
 #include <gsl/gsl_histogram.h>
@@ -14,20 +15,30 @@
 #endif
 #include <libintl.h>
 
+
+#include "gui/preferences.h"
+#include "core/atomic.h"
+
 #define _(String) gettext (String)
 #define gettext_noop(String) String
 #define N_(String) gettext_noop (String)
 
-
 #ifdef SIRIL_OUTPUT_DEBUG
 #define DEBUG_TEST 1
 #else
 #define DEBUG_TEST 0
 #endif
 
-#define siril_debug_print(fmt, ...) \
-            do { if (DEBUG_TEST) fprintf(stderr, fmt, __VA_ARGS__); } while (0)
+#define GLADE_FILE "siril3.glade"
 
+/* https://stackoverflow.com/questions/1644868/define-macro-for-debug-printing-in-c */
+#define siril_debug_print(fmt, ...) \
+   do { if (DEBUG_TEST) fprintf(stdout, fmt, ##__VA_ARGS__); } while (0)
+
+#define PRINT_ALLOC_ERR fprintf(stderr, "Out of memory in %s (%s:%d) - aborting\n", __func__, __FILE__, __LINE__)
+#define PRINT_ANOTHER_THREAD_RUNNING siril_log_message(_("Another task is already in progress, ignoring new request.\n"))
+
+#ifndef RT_INCLUDE
 #undef max
 #define max(a,b) \
    ({ __typeof__ (a) _a = (a); \
@@ -35,38 +46,45 @@
      _a > _b ? _a : _b; })
 
 #undef min
- #define min(a,b) \
+#define min(a,b) \
    ({ __typeof__ (a) _a = (a); \
        __typeof__ (b) _b = (b); \
      _a < _b ? _a : _b; })
 
+#define SWAP(a,b)  { double temp = (a); (a) = (b); (b) = temp; }
+
 #define SQR(x) ((x)*(x))
+#endif
+#define RADCONV ((3600.0 * 180.0) / M_PI) / 1.0E3
 
 #define USHRT_MAX_DOUBLE ((double)USHRT_MAX)
+#define SHRT_MAX_DOUBLE ((double)SHRT_MAX)
 #define USHRT_MAX_SINGLE ((float)USHRT_MAX)
+#define SHRT_MAX_SINGLE ((float)SHRT_MAX)
 #define UCHAR_MAX_DOUBLE ((double)UCHAR_MAX)
 #define UCHAR_MAX_SINGLE ((float)UCHAR_MAX)
+#define INV_USHRT_MAX_SINGLE .000015259022f	// 1/255
+#define INV_UCHAR_MAX_SINGLE .0039215686f	// 1/65535
+
+#define BYTES_IN_A_MB 1048576	// 1024
 
 #define SEQUENCE_DEFAULT_INCLUDE TRUE	// select images by default
 
-typedef unsigned char BYTE;		// default type for image display data
-typedef unsigned short WORD;		// default type for internal image data
+typedef unsigned char BYTE;	// default type for image display data
+typedef unsigned short WORD;	// default type for internal image data
 
-#define MAX_COMMAND_WORDS 16		// max number of words to split in command line input
+#define MAX_SEQPSF 7	// max number of stars for which seqpsf can be run
 
-#define MAX_SEQPSF 7			// max number of stars for which seqpsf can be run
+#define CMD_HISTORY_SIZE 50	// size of the command line history
 
-#define CMD_HISTORY_SIZE 50		// size of the command line history
-
-#define ZOOM_MAX	16.0
-#define ZOOM_MIN	0.0625
+#define ZOOM_MAX	128
+#define ZOOM_MIN	0.03125
+#define ZOOM_IN		1.5
+#define ZOOM_OUT    1.0 / ZOOM_IN
 #define ZOOM_NONE	1.0
 #define ZOOM_FIT	-1.0	// or any value < 0
 #define ZOOM_DEFAULT	ZOOM_FIT
 
-#define LOW_BOUND  0.00002
-#define HIGH_BOUND 0.99998
-
 /* Some statistic constants */
 #define SIGMA_PER_FWHM 2.35482
 #define AVGDEV_NORM 1.2533
@@ -76,6 +94,17 @@ typedef unsigned short WORD;		// default type for internal image data
 #define SN_NORM 1.1926
 #define QN_NORM 2.2191
 
+typedef struct _SirilDialogEntry SirilDialogEntry;
+
+/* used for open and savedialog */
+typedef GtkWidget SirilWidget;
+
+#if (defined _WIN32) || (defined(OS_OSX))
+#define SIRIL_EOL "\r\n"
+#else
+#define SIRIL_EOL "\n"
+#endif
+
 /* when requesting an image redraw, it can be asked to remap its data before redrawing it.
  * REMAP_NONE	doesn't remaps the data,
  * REMAP_ONLY	remaps only the current viewport (color channel) and the mixed (RGB) image
@@ -85,43 +114,21 @@ typedef unsigned short WORD;		// default type for internal image data
 #define REMAP_ONLY	1
 #define REMAP_ALL	2
 
-enum {
-	COLUMN_FILENAME,		// string
-	COLUMN_DATE,		// string
-	N_COLUMNS_CONVERT
-};
-
 typedef enum {
-	TYPEUNDEF=(1 << 1),
-	TYPEFITS= (1 << 2),
-	TYPETIFF= (1 << 3),
+	TYPEUNDEF = (1 << 1),
+	TYPEFITS = (1 << 2),
+	TYPETIFF = (1 << 3),
 	TYPEBMP = (1 << 4),
 	TYPEPNG = (1 << 5),
 	TYPEJPG = (1 << 6),
-	TYPEPNM = (1 << 7),
-	TYPEPIC = (1 << 8),
-	TYPERAW = (1 << 9),
-	TYPEAVI = (1 << 10),
-	TYPESER = (1 << 11),
-	TYPEMP4 = (1 << 12),
-	TYPEWEBM= (1 << 13),
+	TYPEHEIF = (1 << 7),
+	TYPEPNM = (1 << 8),
+	TYPEPIC = (1 << 9),
+	TYPERAW = (1 << 10),
+	TYPEAVI = (1 << 20),
+	TYPESER = (1 << 21),
 } image_type;
 
-#define USE_DARK	0x01
-#define USE_FLAT	0x02
-#define USE_OFFSET	0x04
-#define USE_COSME	0x08	/* cosmetic correction */
-#define USE_OPTD	0x10	/* dark optimization */
-
-/* cookies for the file chooser */
-#define OD_NULL 	0
-#define OD_FLAT 	1
-#define OD_DARK 	2
-#define OD_OFFSET 	3
-#define OD_CWD 		4
-#define OD_OPEN 	5
-#define OD_CONVERT 	6
-
 /* indices of the image data layers */
 #define BW_LAYER 	0
 #define RLAYER		0
@@ -143,26 +150,8 @@ typedef enum {
 #define CP_INIT		0x01	// initialize data array with 0s
 #define CP_ALLOC	0x02	// reallocs data array
 #define CP_COPYA	0x04	// copy data array content
-#define CP_FORMAT	0x08	// copy size and bitpix info
-#define CP_EXTRACT	0x10	// extract a 16bit plane from a 48 bit fit
-#define CP_EXPAND	0x20	// expands a 16bit fits to a 48bit one.
-
-/* processing */
-#define CONVDEBAYER (1 << 1)
-/* SER flags */
-#define CONVDSTFITS (1 << 2)	// assumed as default
-#define CONVDSTSER (1 << 3)
-#define CONVMULTIPLE (1 << 4)
-/* channel conversion type */
-#define CONV1X3	(1 << 6)	// assumed as default
-#define CONV3X1	(1 << 7)
-#define CONV1X1	(1 << 8)
-
-/* operations on image data */
-#define OPER_ADD 'a'
-#define OPER_SUB 's'
-#define OPER_MUL 'm'
-#define OPER_DIV 'd'
+#define CP_FORMAT	0x08	// copy metadata
+#define CP_EXPAND	0x20	// expands a one-channel to a three channels
 
 #define PREVIEW_NB 2
 
@@ -175,41 +164,36 @@ typedef enum {
 #define SCALED_IMAGE -3		// the single image has a different size than
 				// the loaded sequence
 
-#define MAX_STARS 50000		// maximum length of com.stars
+#define MAX_STARS 200000		// maximum length of com.stars
 
-/* constants for loglut function */
-#define LOG 1
-#define EXP -1
+#define INDEX_MAX 100000		// maximum index for images
 
 typedef struct imdata imgdata;
 typedef struct registration_data regdata;
 typedef struct layer_info_struct layer_info;
 typedef struct sequ sequence;
 typedef struct single_image single;
+typedef struct wcs_struct wcs_info;
+typedef struct dft_struct dft_info;
 typedef struct ffit fits;
 typedef struct libraw_config libraw;
 typedef struct phot_config phot;
 typedef struct stack_config stackconf;
+typedef struct comp_config compconf;
 typedef struct cominf cominfo;
 typedef struct image_stats imstats;
 typedef struct rectangle_struct rectangle;
 typedef struct point_struct point;
-typedef struct gradient_struct gradient;
+typedef struct pointf_struct pointf;
+typedef struct pointi_struct pointi;
 typedef struct historic_struct historic;
-typedef struct dateTime_struct dateTime;
 typedef struct fwhm_struct fitted_PSF;
 typedef struct star_finder_struct star_finder_params;
+typedef struct pref_struct preferences;
+typedef struct save_config_struct save_config;
 
 /* global structures */
 
-/* ORDER OF POLYNOMES */
-typedef enum {
-	POLY_1,
-	POLY_2,
-	POLY_3,
-	POLY_4,
-} poly_order;
-
 typedef enum {
 	NORMAL_DISPLAY,	
 	LOG_DISPLAY,
@@ -233,20 +217,21 @@ typedef enum {
 } sliders_mode;
 
 typedef enum {
-	FILE_CONVERSION,
-	IMAGE_SEQ,
-	PRE_PROC,
-	REGISTRATION,
-	PLOT,
-	STACKING,
-	OUTPUT_LOGS
-} main_tabs;
+	OPEN_IMAGE_ERROR = -1,
+	OPEN_IMAGE_OK = 0,
+	OPEN_IMAGE_CANCEL = 10,
+} open_image_status;
 
 typedef enum {
 	BAYER_BILINEAR,
-	BAYER_NEARESNEIGHBOR,
 	BAYER_VNG,
 	BAYER_AHD,
+	BAYER_AMAZE,
+	BAYER_DCB,
+	BAYER_HPHD,
+	BAYER_IGV,
+	BAYER_LMMSE,
+	BAYER_RCD,
 	BAYER_SUPER_PIXEL,
 	XTRANS
 } interpolation_method;
@@ -261,13 +246,13 @@ typedef enum {
 } opencv_interpolation;
 
 typedef enum {
-    BAYER_FILTER_RGGB,
-    BAYER_FILTER_BGGR,
-    BAYER_FILTER_GBRG,
-    BAYER_FILTER_GRBG,
+	BAYER_FILTER_RGGB,
+	BAYER_FILTER_BGGR,
+	BAYER_FILTER_GBRG,
+	BAYER_FILTER_GRBG,
 	XTRANS_FILTER,
-    BAYER_FILTER_NONE = -1		//case where pattern is undefined or untested
-} sensor_pattern ;
+	BAYER_FILTER_NONE = -1		//case where pattern is undefined or untested
+} sensor_pattern;
 #define BAYER_FILTER_MIN BAYER_FILTER_RGGB
 #define BAYER_FILTER_MAX BAYER_FILTER_GRBG
 
@@ -280,8 +265,8 @@ struct layer_info_struct {
 	display_mode rendering_mode;	// defaults to NORMAL_DISPLAY
 };
 
-typedef enum { SEQ_REGULAR, SEQ_SER,
-#if defined(HAVE_FFMS2_1) || defined(HAVE_FFMS2_2)
+typedef enum { SEQ_REGULAR, SEQ_SER, SEQ_FITSEQ,
+#ifdef HAVE_FFMS2
 	SEQ_AVI,
 #endif
 	SEQ_INTERNAL
@@ -291,22 +276,7 @@ typedef enum { SEQ_REGULAR, SEQ_SER,
 struct imdata {
 	int filenum;		/* real file index in the sequence, i.e. for mars9.fit = 9 */
 	gboolean incl;		/* selected in the sequence, included for future processings? */
-	char *date_obs;		/* date of the observation, processed and copied from the header */
-};
-
-/* preprocessing data from GUI */
-struct preprocessing_data {
-	struct timeval t_start;
-	fits *dark, *offset, *flat;
-	gboolean is_sequence;
-	sequence *seq;
-	gboolean autolevel;
-	double sigma[2];
-	gboolean is_cfa;
-	gboolean debayer;
-	gboolean compatibility;
-	float normalisation;
-	int retval;
+	GDateTime *date_obs;/* date of the observation, processed and copied from the header */
 };
 
 /* registration data, exists once for each image and each layer */
@@ -314,7 +284,8 @@ struct registration_data {
 	float shiftx, shifty;	// we could have a subpixel precision, but is it needed? saved
 	fitted_PSF *fwhm_data;	// used in PSF/FWHM registration, not saved
 	float fwhm;		// copy of fwhm->fwhmx, used as quality indicator, saved data
-	float roundness;	// fwhm->fwhmy / fwhm->fwhmx, 0 when uninit, ]0, 1] when set 
+	float weighted_fwhm; // used to exclude spurious images.
+	float roundness;	// fwhm->fwhmy / fwhm->fwhmx, 0 when uninit, ]0, 1] when set
 	double quality;
 };
 
@@ -347,7 +318,8 @@ struct sequ {
 	sequence_type type;
 	struct ser_struct *ser_file;
 	gboolean cfa_opened_monochrome;	// in case the CFA SER was opened in monochrome mode
-#if defined(HAVE_FFMS2_1) || defined(HAVE_FFMS2_2)
+	struct fits_sequence *fitseq_file; // FITS sequence data structure
+#ifdef HAVE_FFMS2
 	struct film_struct *film_file;
 	char *ext;		// extension of video, NULL if not video
 #endif
@@ -357,10 +329,6 @@ struct sequ {
 	omp_lock_t *fd_lock;	// locks for open-mode threaded operations
 #endif
 
-	fits *offset;		// the image containing offset data
-	fits *dark;		// the image containing dark data
-	fits *flat;		// the image containing flat data
-	char *ppprefix;		// prefix for filename output of preprocessing
 	int current;		// file number currently loaded in gfit (or displayed)
 
 	/* registration previsualisation and manual alignment data */
@@ -380,18 +348,32 @@ struct sequ {
 /* this struct is used to manage data associated with a single image loaded, outside a sequence */
 struct single_image {
 	char *filename;		// the name of the file
+	gboolean fileexist;// flag of existing file
 	char *comment;		// comment on how the file got there (user load, result...)
 	int nb_layers;		// number of layers embedded in each image file
 	layer_info *layers;	// info about layers
 	fits *fit;		// the fits is still gfit, but a reference doesn't hurt
-
-	/* enabling pre-processing on a single image */
-	fits *offset;		// the image containing offset data
-	fits *dark;		// the image containing dark data
-	fits *flat;		// the image containing flat data
-	char *ppprefix;		// prefix for filename output of preprocessing
 };
 
+struct wcs_struct {
+	double equinox;
+	double crpix[2];
+	double crval[2];
+	double cdelt[2];
+	double cd[2][2];
+	double crota[2];
+	char objctra[FLEN_VALUE];
+	char objctdec[FLEN_VALUE];
+};
+
+struct dft_struct {
+	double norm[3];			// Normalization value
+	char type[FLEN_VALUE];		// spectrum, phase
+	char ord[FLEN_VALUE];		// regular, centered
+};
+
+typedef enum { DATA_USHORT, DATA_FLOAT, DATA_UNSUPPORTED } data_type;
+
 struct ffit {
 	unsigned int rx;	// image width	(naxes[0])
 	unsigned int ry;	// image height	(naxes[1])
@@ -400,7 +382,7 @@ struct ffit {
 	/* bitpix can take the following values:
 	 * BYTE_IMG	(8-bit byte pixels, 0 - 255)
 	 * SHORT_IMG	(16 bit signed integer pixels)	
-	 * USHORT_IMG	(16 bit unsigned integer pixels)	(used by Siril, quite off-standard)
+	 * USHORT_IMG	(16 bit unsigned integer pixels)	(used by Siril, a bit unusual)
 	 * LONG_IMG	(32-bit integer pixels)
 	 * FLOAT_IMG	(32-bit floating point pixels)
 	 * DOUBLE_IMG	(64-bit floating point pixels)
@@ -414,34 +396,49 @@ struct ffit {
 	 * */
 
 	/* data obtained from the FITS file */
+	char *header;	// entire header of the FITS file. NULL for non-FITS file.
 	WORD lo;	// MIPS-LO key in FITS file, which is "Lower visualization cutoff"
 	WORD hi;	// MIPS-HI key in FITS file, which is "Upper visualization cutoff"
+	double data_max; // used to check if 32b float is in the [0, 1] range
 	float pixel_size_x, pixel_size_y;	// XPIXSZ and YPIXSZ keys
 	unsigned int binning_x, binning_y;		// XBINNING and YBINNING keys
-	char date_obs[FLEN_VALUE];		// YYYY-MM-DDThh:mm:ss observation start, UT
-	char date[FLEN_VALUE];		// YYYY-MM-DDThh:mm:ss creation of file, UT
+	gboolean unbinned;
+	char row_order[FLEN_VALUE];
+	GDateTime *date, *date_obs;
+	double expstart, expend;
 	char instrume[FLEN_VALUE];		// INSTRUME key
 	char telescop[FLEN_VALUE];		// TELESCOP key
 	char observer[FLEN_VALUE];		// OBSERVER key
 	char bayer_pattern[FLEN_VALUE];	// BAYERPAT key Bayer Pattern if available
+	int bayer_xoffset, bayer_yoffset;
 	/* data obtained from FITS or RAW files */
 	double focal_length, iso_speed, exposure, aperture, ccd_temp;
 	double cvf; // Conversion factor (e-/adu)
 
+	/* Plate Solving data */
+	wcs_info wcsdata;	// data from the header
+#ifdef HAVE_WCSLIB
+	struct wcsprm *wcslib;	// struct of the lib
+#endif
+
 	/* data used in the Fourier space */
-	double dft_norm[3];			// Normalization value
-	char dft_type[FLEN_VALUE];		// spectrum, phase
-	char dft_ord[FLEN_VALUE];		// regular, centered
-	unsigned int dft_rx, dft_ry;		// padding: original value of picture size
+	dft_info dft;
 	
 	/* data computed or set by Siril */
 	imstats **stats;	// stats of fit for each layer, null if naxes[2] is unknown
-	double mini, maxi;	// max of the stats->max[3]
+	double mini, maxi;	// min and max of the stats->max[3]
 
 	fitsfile *fptr;		// file descriptor. Only used for file read and write.
+
+	data_type type;		// use of data or fdata is managed by this
 	WORD *data;		// 16-bit image data (depending on image type)
 	WORD *pdata[3];		// pointers on data, per layer data access (RGB)
-	char *header;		// entire header of the FITS file. NULL for non-FITS file.
+	float *fdata;		// same with float
+	float *fpdata[3];	// same with float
+
+	gboolean top_down;	// image data is stored top-down, normally false for FITS, true for SER
+
+	GSList *history;	// Former HISTORY comments of FITS file
 };
 
 /* This structure is used for all the elements in the box libraw_settings.
@@ -460,6 +457,7 @@ struct phot_config {
 	double gain;	// A/D converter gain in electrons per ADU
 	double inner;	// Inner radius of the annulus used to measure local background.
 	double outer;	// Outer radius of the annulus used to measure local background.
+	int minval, maxval;
 };
 
 struct debayer_config {
@@ -467,14 +465,27 @@ struct debayer_config {
 	gboolean use_bayer_header;		// use the pattern given in the file header
 	sensor_pattern bayer_pattern;		// user-defined Bayer pattern
 	interpolation_method bayer_inter;	// interpolation method for non-libraw debayer
-	gboolean compatibility;				// ensure KSTARS compatibility if TRUE
+	gboolean top_down;				// debayer top-down orientation
+	int xbayeroff, ybayeroff;			// x and y Bayer offsets
 };
 
 struct stack_config {
 	int method;				// 0=sum, 1=median, 2=average, 3=pixel max, 4=pixel min - Use to save preferences in the init file
 	int normalisation_method;
 	int rej_method;
-	double memory_percent;			// percent of available memory to use for stacking
+	double sigma_low, sigma_high;
+	double linear_low, linear_high;
+	double percentile_low, percentile_high;
+	enum { RATIO, AMOUNT, UNLIMITED } mem_mode; // mode of memory management
+	double memory_ratio;			// ratio of available memory to use for stacking (and others)
+	double memory_amount;			// amount of memory in GB to use for stacking (and others)
+};
+
+struct comp_config {
+	gboolean fits_enabled;		// true enabled
+	int fits_method;		// 0=Rice, 1=GZIP1, 2=GZIP2, 3=Hcompress
+	double fits_quantization;	// quantization factor for floating point compression
+	double fits_hcompress_scale;		// scale factor for Hcompress compression
 };
 
 struct rectangle_struct {
@@ -485,25 +496,19 @@ struct point_struct {
 	double x, y;
 };
 
-struct gradient_struct {
-	point centre;
-	double boxvalue[3];
+struct pointf_struct {
+	float x, y;
+};
+
+struct pointi_struct {
+	int x, y;
 };
 
 struct historic_struct {
 	char *filename;
 	char history[FLEN_VALUE];
 	int rx, ry;
-};
-
-struct dateTime_struct {
-	int year;
-	int month;
-	int day;
-	int hour;
-	int min;
-	int sec;
-	int ms;
+	data_type type;
 };
 
 struct star_finder_struct {
@@ -512,6 +517,71 @@ struct star_finder_struct {
 	double roundness;
 };
 
+struct save_config_struct {
+	gboolean quit;
+	gboolean warn_script;
+};
+
+/**
+ * This is the preference structure.
+ * WARNING!!
+ * If you update something in this structure you absolutely need to
+ * update pref_init in preferences.c
+ */
+struct pref_struct {
+	gboolean first_start; // use to display information at first use
+	/* state of window */
+	gboolean remember_windows;
+	rectangle main_w_pos;
+	gboolean is_maximized;
+
+	gboolean prepro_cfa;	// Use to save type of sensor for cosmetic correction in preprocessing
+	gboolean prepro_equalize_cfa;  // Use to save if flat will be equalized in preprocessing
+	gboolean fix_xtrans; // Use to fix xtrans darks and biases with the AF square
+	rectangle xtrans_af; // if no xtrans model found, use these values
+	rectangle xtrans_sample; // if no xtrans model found, use these values
+	gchar *prepro_bias_lib;
+	gboolean use_bias_lib;
+	gchar *prepro_dark_lib;
+	gboolean use_dark_lib;
+	gchar *prepro_flat_lib;
+	gboolean use_flat_lib;
+
+	save_config save;
+	gboolean show_thumbnails; // show or don't show thumbnails in open dialog box
+	gint thumbnail_size;
+	gboolean check_update; // check update at startup
+	gboolean script_check_requires; // check the requires command in scripts
+
+	gint combo_theme;           // value of the combobox theme
+	gdouble font_scale;           // font scale
+	gboolean icon_symbolic;		// icon style
+	gchar *combo_lang;           // string value of the combobox lang
+
+	gchar *ext;		// FITS extension used in SIRIL
+
+	gchar *swap_dir;		// swap directory
+	GSList *script_path;	// script path directories
+
+	libraw raw_set;			// the libraw settings
+	struct debayer_config debayer;	// debayer settings
+	phot phot_set;          // photometry settings
+	gboolean catalog[6]; // Yet 6 catalogs
+
+	stackconf stack; // stacking option
+	compconf comp; // compression option
+	gboolean rgb_aladin; // Add CTYPE3='RGB' in the FITS header
+
+	gboolean force_to_16bit;
+
+	gint selection_guides;	// number of elements of the grid guides (2 for a simple cross, 3 for the 3 thirds rule, etc.)
+
+	gchar *copyright;		// User copyright when saving image as TIFF
+};
+/**
+ * End of preference structure. Read above.
+ */
+
 /* The global data structure of siril, the only with gfit and the gtk builder,
  * declared in main.c */
 struct cominf {
@@ -524,7 +594,8 @@ struct cominf {
 	int surface_height[MAXVPORT];	// allocated height
 	cairo_surface_t *surface[MAXVPORT];
 	gboolean buf_is_dirty[MAXVPORT];// dirtyness of each buffer (= need to redraw)
-	
+	GdkPixbuf *siril_pix; // Siril.svg image loaded at initialization
+
 	/* Color map */
 	color_map color;
 
@@ -533,15 +604,23 @@ struct cominf {
 	GtkAdjustment *hadj[MAXVPORT];	// adjustments of vport scrollbars
 	GtkAdjustment *vadj[MAXVPORT];	// adjustments of vport scrollbars
 	sliders_mode sliders;		// 0: min/max, 1: MIPS-LO/HI, 2: user
-	int preprostatus;
-	gboolean prepro_cfa;	// Use to save type of sensor for cosmetic correction in preprocessing
 	gboolean show_excluded;		// show excluded images in sequences
+
+	cairo_matrix_t display_matrix; // matrix used for image rendering (convert image to display coordinates)
+	cairo_matrix_t image_matrix; // inverse of display_matrix (convert display to image coordinates)
 	double zoom_value;		// 1.0 is normal zoom, use get_zoom_val() to access it
+	point display_offset; // image display offset
+	gboolean translating;		// true we are in display transating mode
+
+	preferences pref; // saved variable in preferences
 
 	/* selection rectangle for registration, FWHM, PSF */
-	gboolean drawing;		// true if the rectangle is being set (clicked motion)
-	gint startX, startY;		// where the mouse was originally clicked to
+	gboolean drawing;			// true if the rectangle is being set (clicked motion)
+	pointi start;				// where the mouse was originally clicked to
+	pointi origin;				// where the selection was originally located
+	gboolean freezeX, freezeY;	// locked axis during modification of a selection 
 	rectangle selection;		// coordinates of the selection rectangle
+	double ratio;				// enforced ratio of the selection (default is 0: none)
 
 	/* alignment preview data */
 	//guchar *preview_buf[PREVIEW_NB];
@@ -551,17 +630,11 @@ struct cominf {
 	cairo_surface_t *refimage_surface;
 
 	gchar *wd;			// working directory, where images and sequences are
-	gchar *initfile;			// the path of the init file
+	gchar *initfile;	// the path of the init file
 	
-	char *ext;		// FITS extension used in SIRIL
-
 	int reg_settings;		// Use to save registration method in the init file
-	
-	gboolean dontShowConfirm;
 
-	gboolean have_dark_theme;	// we have a dark theme, use bright colours
-
-	stackconf stack;
+	gboolean cache_upscaled;	// keep up-scaled files for 'drizzle' (only used by developers)
 	
 	int filter;			// file extension filter for open/save dialogs
 
@@ -579,12 +652,6 @@ struct cominf {
 	int hist_size;			// allocated size
 	int hist_current;		// current index
 	int hist_display;		// displayed index
-	gchar *swap_dir;		// swap directory
-	GSList *script_path;	// script path directories
-
-	libraw raw_set;			// the libraw settings
-	struct debayer_config debayer;	// debayer settings
-	phot phot_set;          // photometry settings
 
 	sequence seq;			// currently loaded sequence	TODO: *seq
 	single *uniq;			// currently loaded image, if outside sequence
@@ -597,9 +664,8 @@ struct cominf {
 	int selected_star;		// current selected star in the GtkListStore
 	double magOffset;		// offset to reduce the real magnitude, single image
 	
-	gradient *grad;
-	int grad_nb_boxes, grad_size_boxes;
-	gboolean grad_boxes_drawn;
+	GSList *grad_samples;
+	GSList *found_object;
 
 	int max_thread;			// maximum of thread used for parallel execution
 
@@ -620,7 +686,7 @@ struct image_stats {
 	double mean, median, sigma, avgDev, mad, sqrtbwmv,
 	       location, scale, min, max, normValue, bgnoise;
 
-	int _nb_refs;	// reference counting for data management
+	atomic_int* _nb_refs;	// reference counting for data management
 };
 
 typedef struct Homo {
@@ -655,8 +721,6 @@ struct image_layer_struct {
 extern GtkBuilder *builder;	// get widget references anywhere
 extern cominfo com;		// the main data struct
 extern fits gfit;		// currently loaded image
-extern char **supported_extensions;
-extern char *filter_pattern[];
 #endif
 
 #endif /*SIRIL */
