@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -31,18 +31,25 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
-#include "gui/callbacks.h"
+#include "core/siril_date.h"
+#include "core/OS_utils.h"
+#include "core/processing.h"
+#include "gui/utils.h"
+#include "gui/image_display.h"
+#include "gui/message_dialog.h"
 #include "gui/progress_and_log.h"
+#include "gui/sequence_list.h"
 #include "kplot.h"
 #include "algos/PSF.h"
 #include "io/ser.h"
 #include "gui/gnuplot_i/gnuplot_i.h"
+#include "gui/PSF_list.h"
 
 #define XLABELSIZE 15
 
 static GtkWidget *drawingPlot = NULL, *sourceCombo = NULL, *combo = NULL,
-		*varCurve = NULL, *buttonExport = NULL, *buttonClearAll = NULL,
-		*buttonClearLatest = NULL;
+		*varCurve = NULL, *buttonClearAll = NULL,
+		*buttonClearLatest = NULL, *arcsec = NULL, *julianw = NULL;
 static pldata *plot_data;
 static struct kpair ref;
 static gboolean is_fwhm = FALSE, use_photometry = FALSE, requires_color_update =
@@ -52,22 +59,45 @@ static gchar *xlabel = NULL;
 static enum photmetry_source selected_source = ROUNDNESS;
 static int julian0 = 0;
 static gnuplot_ctrl *gplot = NULL;
+static gboolean is_arcsec = FALSE;
+static gboolean force_Julian = FALSE;
 
 static void update_ylabel();
 static void set_colors(struct kplotcfg *cfg);
-void on_GtkEntryCSV_changed(GtkEditable *editable, gpointer user_data);
+static void free_colors(struct kplotcfg *cfg);
 
 static pldata *alloc_plot_data(int size) {
 	pldata *plot = calloc(1, sizeof(pldata));
-	if (!plot)
+	if (!plot) {
+		PRINT_ALLOC_ERR;
 		return NULL;
+	}
+	plot->frame = calloc(size, sizeof(double));
+	if (!plot->frame) {
+		PRINT_ALLOC_ERR;
+		free(plot);
+		return NULL;
+	}
+	plot->julian = calloc(size, sizeof(double));
+	if (!plot->julian) {
+		PRINT_ALLOC_ERR;
+		free(plot->frame);
+		free(plot);
+		return NULL;
+	}
 	plot->data = calloc(size, sizeof(struct kpair));
 	if (!plot->data) {
+		PRINT_ALLOC_ERR;
+		free(plot->frame);
+		free(plot->julian);
 		free(plot);
 		return NULL;
 	}
 	plot->err = calloc(size, sizeof(struct kpair));
 	if (!plot->err) {
+		PRINT_ALLOC_ERR;
+		free(plot->frame);
+		free(plot->julian);
 		free(plot->data);
 		free(plot);
 		return NULL;
@@ -84,73 +114,48 @@ static void build_registration_dataset(sequence *seq, int layer, int ref_image,
 	for (i = 0, j = 0; i < plot->nb; i++) {
 		if (!seq->imgparam[i].incl)
 			continue;
-		plot->data[j].x = (double) i;
+		plot->data[j].x = (double) i + 1;
 		plot->data[j].y = is_fwhm ?	seq->regparam[layer][i].fwhm :
 						seq->regparam[layer][i].quality;
+		plot->frame[j] =  plot->data[j].x;
 		j++;
 	}
 	plot->nb = j;
 
-	ref.x = (double) ref_image;
+	ref.x = (double) ref_image + 1;
 	ref.y = is_fwhm ?
 			seq->regparam[layer][ref_image].fwhm :
 			seq->regparam[layer][ref_image].quality;
 
 }
 
-static const uint64_t epochTicks = 621355968000000000UL;
+static void set_x_values(sequence *seq, pldata *plot, int i, int j) {
+	if (seq->imgparam[i].date_obs) {
+		double julian;
+		GDateTime *tsi = g_date_time_ref(seq->imgparam[i].date_obs);
+		if (seq->exposure) {
+			GDateTime *new_dt = g_date_time_add_seconds(tsi, seq->exposure / 2.0);
+			julian = date_time_to_Julian(new_dt);
+			g_date_time_unref(new_dt);
+		} else {
+			julian = date_time_to_Julian(tsi);
+		}
 
-static double serTimestamp_toJulian(uint64_t timestamp) {
-	struct tm *t;
-	dateTime dt;
-	uint64_t t1970_ms = (timestamp - epochTicks) / 10000;
-	time_t secs = t1970_ms / 1000;
-	int ms = t1970_ms % 1000;
-#ifdef HAVE_GMTIME_R
-	struct tm t_;
-#endif
+		plot->julian[j] = julian - (double) julian0;
 
-#ifdef _WIN32
-	t = gmtime (&secs);
-#else
-#ifdef HAVE_GMTIME_R
-	t = gmtime_r (&secs, &t_);
-#else
-	t = gmtime(&secs);
-#endif /* HAVE_GMTIME_R */
-#endif /* _WIN32 */
+		g_date_time_unref(tsi);
+	} else {
+		plot->julian[j] = (double) i + 1; // should not happen.
+	}
+	plot->frame[j] = (double) i + 1;
 
-	/* If the gmtime() call has failed, "secs" is too big. */
-	if (t == NULL) {
-		return -1.0;
+	if (julian0 && force_Julian) {
+		plot->data[j].x = plot->julian[j];
+	} else {
+		plot->data[j].x = plot->frame[j];
 	}
 
-	/* Get real year and month */
-	memset(&dt, 0, sizeof(dateTime));
-	dt.year = t->tm_year + 1900;
-	dt.month = t->tm_mon + 1;
-	dt.day = t->tm_mday;
-	dt.hour = t->tm_hour;
-	dt.min = t->tm_min;
-	dt.sec = t->tm_sec;
-	dt.ms = ms;
-
-	return encodeJD(dt);
-}
-
-static double dateTimestamp_toJulian(char *timestamp, double exp) {
-	dateTime dt;
-
-	if (timestamp[0] == '\0')
-		return -1;
-
-	memset(&dt, 0, sizeof(dateTime));
-	sscanf(timestamp, "%04d-%02d-%02dT%02d:%02d:%02d.%02d", &dt.year, &dt.month, &dt.day,
-			&dt.hour, &dt.min, &dt.sec, &dt.ms);
-	/* we add exposure / 2 to the timestamp */
-	dt.sec += exp / 2.0;
-
-	return encodeJD(dt);
+	plot->err[j].x = plot->data[j].x;
 }
 
 static void build_photometry_dataset(sequence *seq, int dataset, int size,
@@ -165,42 +170,35 @@ static void build_photometry_dataset(sequence *seq, int dataset, int size,
 		if (!seq->imgparam[i].incl || !psfs[i])
 			continue;
 		if (!julian0 && !xlabel) {
-			/* X axis init */
-			if (seq->type == SEQ_SER && seq->ser_file->ts
-					&& seq->ser_file->ts_max > seq->ser_file->ts_min) {
-				/* Get SER start date */
-				julian0 = (int) serTimestamp_toJulian(seq->ser_file->ts[i]);
-			} else if (seq->type == SEQ_REGULAR && seq->imgparam[i].date_obs) {
-				/* Get FITS start date */
-				char *ts0 = seq->imgparam[i].date_obs;
-				julian0 = (int) dateTimestamp_toJulian(ts0, seq->exposure);
+			if (seq->imgparam[i].date_obs) {
+				GDateTime *ts0 = g_date_time_ref(seq->imgparam[i].date_obs);
+				if (seq->exposure) {
+					GDateTime *new_dt = g_date_time_add_seconds(ts0, seq->exposure / 2.0);
+					julian0 = (int) date_time_to_Julian(new_dt);
+					g_date_time_unref(new_dt);
+				} else {
+					julian0 = (int) date_time_to_Julian(ts0);
+				}
+				g_date_time_unref(ts0);
 			}
-			if (julian0) {
+			if (julian0 && force_Julian) {
 				xlabel = calloc(XLABELSIZE, sizeof(char));
 				g_snprintf(xlabel, XLABELSIZE, "(JD) %d +", julian0);
 			} else {
 				xlabel = g_strdup(_("Frames"));
 			}
 		}
-
-		if (julian0 && seq->type == SEQ_SER && seq->ser_file->ts
-				&& seq->ser_file->ts_max > seq->ser_file->ts_min) {
-			double julian = serTimestamp_toJulian(seq->ser_file->ts[i]);
-			plot->data[j].x = julian - (double)julian0;
-		} else if (julian0 && seq->type == SEQ_REGULAR && seq->imgparam[i].date_obs) {
-			char *tsi = seq->imgparam[i].date_obs;
-			double julian = dateTimestamp_toJulian(tsi, seq->exposure);
-			plot->data[j].x = julian - (double)julian0;
-		} else {
-			plot->data[j].x = (double) i;
-		}
-		plot->err[j].x = plot->data[j].x;
+		set_x_values(seq, plot, i, j);
 
 		switch (selected_source) {
 			case ROUNDNESS:
 				plot->data[j].y = psfs[i]->fwhmy / psfs[i]->fwhmx;
 				break;
 			case FWHM:
+				if (is_arcsec)
+					fwhm_to_arcsec_if_needed(&gfit, psfs[i]);
+				else
+					fwhm_to_pixels(psfs[i]);
 				plot->data[j].y = psfs[i]->fwhmx;
 				break;
 			case AMPLITUDE:
@@ -209,6 +207,7 @@ static void build_photometry_dataset(sequence *seq, int dataset, int size,
 			case MAGNITUDE:
 				plot->data[j].y = psfs[i]->mag;
 				plot->err[j].y = psfs[i]->s_mag;
+
 				if (seq->reference_star >= 0) {
 					/* we have a reference star for the sequence,
 					 * with photometry data */
@@ -268,17 +267,17 @@ static gboolean gnuplot_is_available() {
 }
 #endif
 
-static int lightCurve(pldata *plot, sequence *seq) {
+static int lightCurve(pldata *plot, sequence *seq, gchar *filename) {
 	int i, j, k, nbImages = 0, ret = 0;
 	pldata *tmp_plot = plot;
 	double *vmag, *err, *x, *real_x;
 
 	if (!gnuplot_is_available()) {
-		char *msg = siril_log_message(_("Gnuplot is unavailable. "
-				"Please consider to install it before "
+		char *msg = siril_log_message(_("Please consider to install it before "
 				"trying to plot a graph of a variable star.\n"));
 
-		show_dialog(msg, _("Warning"), "dialog-warning-symbolic");
+		siril_message_dialog( GTK_MESSAGE_WARNING, _("Gnuplot is unavailable"), msg);
+
 		return -1;
 	}
 
@@ -294,6 +293,10 @@ static int lightCurve(pldata *plot, sequence *seq) {
 	err = calloc(nbImages, sizeof(double));
 	x = calloc(nbImages, sizeof(double));
 	real_x = calloc(nbImages, sizeof(double));
+	if (!vmag || !err || !x || !real_x) {
+		PRINT_ALLOC_ERR;
+		return -1;
+	}
 	for (i = 0, j = 0; i < plot->nb; i++) {
 		if (!seq->imgparam[i].incl)
 			continue;
@@ -308,6 +311,7 @@ static int lightCurve(pldata *plot, sequence *seq) {
 			if (tmp_plot->err[k].x == tmp_plot->data[j].x)
 				err[j] = tmp_plot->err[k].y;
 		}
+
 		x[j] = tmp_plot->data[j].x;
 		real_x[j] = x[j] + (double) julian0;
 		tmp_plot = tmp_plot->next;
@@ -365,21 +369,11 @@ static int lightCurve(pldata *plot, sequence *seq) {
 	gnuplot_plot_xyyerr(gplot, x, vmag, err, nbImages, "");
 
 	/* Exporting data in a dat file */
-	GtkEntry *EntryCSV = GTK_ENTRY(lookup_widget("GtkEntryCSV"));
-	const gchar *file = gtk_entry_get_text(EntryCSV);
-	if (file && file[0] != '\0') {
-		gchar *filename = g_strndup(file, strlen(file) + 5);
-		g_strlcat(filename, ".dat", strlen(file) + 5);
-		ret = gnuplot_write_xyyerr_dat(filename, real_x, vmag, err, nbImages,
-				"JD_UT V-C err");
-		if (!ret) {
-			char *msg = siril_log_message(_("%s has been saved.\n"), filename);
-			show_dialog(msg, _("Information"), "dialog-information-symbolic");
-		} else {
-			show_dialog(_("Something went wrong while saving plot"), _("Error"),
-					"dialog-error-symbolic");
-		}
-		g_free(filename);
+	ret = gnuplot_write_xyyerr_dat(filename, real_x, vmag, err, nbImages, "JD_UT V-C err");
+	if (!ret) {
+		siril_log_message(_("%s has been saved.\n"), filename);
+	} else {
+		siril_message_dialog( GTK_MESSAGE_ERROR, _("Error"), _("Something went wrong while saving plot"));
 	}
 
 	free(vmag);
@@ -388,56 +382,93 @@ static int lightCurve(pldata *plot, sequence *seq) {
 	return 0;
 }
 
-static int exportCSV(pldata *plot, sequence *seq) {
-	int i, j, ret = 0;
-	const gchar *file;
-	gchar *filename, *msg;
-	GtkEntry *EntryCSV;
+static int exportCSV(pldata *plot, sequence *seq, gchar *filename) {
+	GError *error = NULL;
 
-	EntryCSV = GTK_ENTRY(lookup_widget("GtkEntryCSV"));
-	file = gtk_entry_get_text(EntryCSV);
-	if (file && file[0] != '\0') {
-		filename = g_strndup(file, strlen(file) + 5);
-		g_strlcat(filename, ".csv", strlen(file) + 5);
-		FILE *csv = g_fopen(filename, "w");
-		if (csv == NULL) {
-			ret = 1;
-		} else {
-			if (use_photometry) {
-				pldata *tmp_plot = plot;
-				for (i = 0, j = 0; i < plot->nb; i++) {
-					if (!seq->imgparam[i].incl)
-						continue;
-					int x = 0;
-					fprintf(csv, "%g", tmp_plot->data[j].x);
-					while (x < MAX_SEQPSF && seq->photometry[x]) {
-						fprintf(csv, ", %g", tmp_plot->data[j].y);
-						tmp_plot = tmp_plot->next;
-						++x;
-					}
-					fprintf(csv, "\n");
-					tmp_plot = plot;
-					j++;
-				}
-			} else {
-				for (i = 0, j = 0; i < plot->nb; i++) {
-					if (!seq->imgparam[i].incl)
-						continue;
-					fprintf(csv, "%g, %g\n", plot->data[j].x, plot->data[j].y);
-					j++;
-				}
+	GFile *file = g_file_new_for_path(filename);
+	GOutputStream *output_stream = (GOutputStream*) g_file_replace(file, NULL, FALSE,
+			G_FILE_CREATE_NONE, NULL, &error);
+
+	if (output_stream == NULL) {
+		if (error != NULL) {
+			g_warning("%s\n", error->message);
+			g_clear_error(&error);
+			fprintf(stderr, "exportCSV: Cannot export\n");
+		}
+		g_object_unref(file);
+		return 1;
+	}
+
+	if (use_photometry) {
+		pldata *tmp_plot = plot;
+		for (int i = 0, j = 0; i < plot->nb; i++) {
+			if (!seq->imgparam[i].incl)
+				continue;
+			int x = 0;
+			double date = tmp_plot->data[j].x;
+			if (julian0 && force_Julian) {
+				date += julian0;
 			}
-			g_free(filename);
-			fclose(csv);
+			gchar *buffer = g_strdup_printf("%.10lf", date);
+			if (!g_output_stream_write_all(output_stream, buffer, strlen(buffer), NULL, NULL, &error)) {
+				g_warning("%s\n", error->message);
+				g_free(buffer);
+				g_clear_error(&error);
+				g_object_unref(output_stream);
+				g_object_unref(file);
+				return 1;
+			}
+			g_free(buffer);
+			while (x < MAX_SEQPSF && seq->photometry[x]) {
+				buffer = g_strdup_printf(", %g", tmp_plot->data[j].y);
+				if (!g_output_stream_write_all(output_stream, buffer, strlen(buffer), NULL, NULL, &error)) {
+					g_warning("%s\n", error->message);
+					g_free(buffer);
+					g_clear_error(&error);
+					g_object_unref(output_stream);
+					g_object_unref(file);
+					return 1;
+				}
+				tmp_plot = tmp_plot->next;
+				++x;
+				g_free(buffer);
+			}
+			if (!g_output_stream_write_all(output_stream, "\n", 1, NULL, NULL, &error)) {
+				g_warning("%s\n", error->message);
+				g_free(buffer);
+				g_clear_error(&error);
+				g_object_unref(output_stream);
+				g_object_unref(file);
+				return 1;
+			}
+			tmp_plot = plot;
+			j++;
+		}
+	} else {
+		for (int i = 0, j = 0; i < plot->nb; i++) {
+			if (!seq->imgparam[i].incl)
+				continue;
+			double date = plot->data[j].x;
+			if (julian0) {
+				date += julian0;
+			}
+			gchar *buffer = g_strdup_printf("%.10lf, %g\n", date, plot->data[j].y);
+			if (!g_output_stream_write_all(output_stream, buffer, strlen(buffer), NULL, NULL, &error)) {
+				g_warning("%s\n", error->message);
+				g_free(buffer);
+				g_clear_error(&error);
+				g_object_unref(output_stream);
+				g_object_unref(file);
+				return 1;
+			}
+			j++;
+			g_free(buffer);
 		}
 	}
-	if (!ret) {
-		msg = siril_log_message(_("%s.csv has been saved.\n"), file);
-		show_dialog(msg, _("Information"), "dialog-information-symbolic");
-	} else {
-		show_dialog(_("Something went wrong while saving plot"), _("Error"),
-				"dialog-error-symbolic");
-	}
+	siril_log_message(_("%s has been saved.\n"), filename);
+	g_object_unref(output_stream);
+	g_object_unref(file);
+
 	return 0;
 }
 
@@ -445,6 +476,10 @@ static void free_plot_data() {
 	pldata *plot = plot_data;
 	while (plot) {
 		pldata *next = plot->next;
+		if (plot->julian)
+			free(plot->julian);
+		if (plot->frame)
+			free(plot->frame);
 		if (plot->data)
 			free(plot->data);
 		if (plot->err)
@@ -464,17 +499,11 @@ void on_plotSourceCombo_changed(GtkComboBox *box, gpointer user_data) {
 	use_photometry = gtk_combo_box_get_active(GTK_COMBO_BOX(box));
 	gtk_widget_set_visible(combo, use_photometry);
 	gtk_widget_set_visible(varCurve, use_photometry);
+	gtk_widget_set_visible(arcsec, use_photometry);
+	gtk_widget_set_visible(julianw, use_photometry);
 	drawPlot();
 }
 
-void on_GtkEntryCSV_changed(GtkEditable *editable, gpointer user_data) {
-	const gchar *txt;
-	if (!buttonExport)
-		return;
-	txt = gtk_entry_get_text(GTK_ENTRY(editable));
-	gtk_widget_set_sensitive(buttonExport, txt[0] != '\0' && plot_data);
-}
-
 void reset_plot() {
 	free_plot_data();
 	if (sourceCombo) {
@@ -482,7 +511,8 @@ void reset_plot() {
 		gtk_widget_set_visible(sourceCombo, FALSE);
 		gtk_widget_set_visible(combo, FALSE);
 		gtk_widget_set_visible(varCurve, FALSE);
-		gtk_widget_set_sensitive(buttonExport, FALSE);
+		gtk_widget_set_visible(arcsec, FALSE);
+		gtk_widget_set_visible(julianw, FALSE);
 		gtk_widget_set_sensitive(buttonClearLatest, FALSE);
 		gtk_widget_set_sensitive(buttonClearAll, FALSE);
 	}
@@ -508,8 +538,9 @@ void drawPlot() {
 		drawingPlot = lookup_widget("DrawingPlot");
 		combo = lookup_widget("plotCombo");
 		varCurve = lookup_widget("varCurvePhotometry");
+		arcsec = lookup_widget("arcsecPhotometry");
+		julianw = lookup_widget("JulianPhotometry");
 		sourceCombo = lookup_widget("plotSourceCombo");
-		buttonExport = lookup_widget("ButtonSaveCSV");
 		buttonClearAll = lookup_widget("clearAllPhotometry");
 		buttonClearLatest = lookup_widget("clearLastPhotometry");
 	}
@@ -568,25 +599,58 @@ void drawPlot() {
 
 		build_registration_dataset(seq, layer, ref_image, plot_data);
 	}
-	on_GtkEntryCSV_changed(GTK_EDITABLE(lookup_widget("GtkEntryCSV")), NULL);
+	gtk_widget_set_sensitive(julianw, julian0);
 	gtk_widget_queue_draw(drawingPlot);
 }
 
+static void set_filter(GtkFileChooser *dialog, const gchar *format) {
+	GtkFileFilter *f = gtk_file_filter_new();
+	gchar *name = g_strdup_printf(_("Output files (*%s)"), format);
+	gchar *pattern = g_strdup_printf("*%s", format);
+	gtk_file_filter_set_name(f, name);
+	gtk_file_filter_add_pattern(f, pattern);
+	gtk_file_chooser_add_filter(dialog, f);
+	gtk_file_chooser_set_filter(dialog, f);
+
+	g_free(name);
+	g_free(pattern);
+}
+
+static void save_dialog(const gchar *format, int (export_function)(pldata *, sequence *, gchar *)) {
+	GtkWindow *control_window = GTK_WINDOW(lookup_widget("control_window"));
+	SirilWidget *widgetdialog = siril_file_chooser_save(control_window, GTK_FILE_CHOOSER_ACTION_SAVE);
+	GtkFileChooser *dialog = GTK_FILE_CHOOSER(widgetdialog);
+
+	gtk_file_chooser_set_current_folder(dialog, com.wd);
+	gtk_file_chooser_set_select_multiple(dialog, FALSE);
+	gtk_file_chooser_set_do_overwrite_confirmation(dialog, TRUE);
+	gtk_file_chooser_set_current_name(dialog, format);
+	set_filter(dialog, format);
+
+	gint res = siril_dialog_run(widgetdialog);
+	if (res == GTK_RESPONSE_ACCEPT) {
+		gchar *file = gtk_file_chooser_get_filename(dialog);
+		export_function(plot_data, &com.seq, file);
+
+		g_free(file);
+	}
+	siril_widget_destroy(widgetdialog);
+}
+
 void on_ButtonSaveCSV_clicked(GtkButton *button, gpointer user_data) {
 	set_cursor_waiting(TRUE);
-	exportCSV(plot_data, &com.seq);
+	save_dialog(".csv", exportCSV);
 	set_cursor_waiting(FALSE);
 }
 
 void on_varCurvePhotometry_clicked(GtkButton *button, gpointer user_data) {
 	set_cursor_waiting(TRUE);
-	lightCurve(plot_data, &com.seq);
+	save_dialog(".dat", lightCurve);
 	set_cursor_waiting(FALSE);
 }
 
 void free_photometry_set(sequence *seq, int set) {
-	int j;
-	for (j = 0; j < seq->number; j++) {
+	for (int j = 0; j < seq->number; j++) {
 		if (seq->photometry[set][j])
 			free(seq->photometry[set][j]);
 	}
@@ -602,24 +666,26 @@ void on_clearLatestPhotometry_clicked(GtkButton *button, gpointer user_data) {
 		i--;
 		free_photometry_set(&com.seq, i);
 	}
-	if (i == 0)
+	if (i == 0) {
 		reset_plot();
+		clear_stars_list();
+	}
 	drawPlot();
 }
 
 void on_clearAllPhotometry_clicked(GtkButton *button, gpointer user_data) {
-	int i;
-	for (i = 0; i < MAX_SEQPSF && com.seq.photometry[i]; i++) {
+	for (int i = 0; i < MAX_SEQPSF && com.seq.photometry[i]; i++) {
 		free_photometry_set(&com.seq, i);
 	}
 	reset_plot();
+	clear_stars_list();
 	drawPlot();
 }
 
 gboolean on_DrawingPlot_draw(GtkWidget *widget, cairo_t *cr, gpointer data) {
 	guint width, height, i, j;
-	double mean;
-	int min, max, nb_graphs = 0;
+	double mean, color;
+	int nb_graphs = 0;
 	struct kpair *avg;
 	struct kplotcfg cfgplot;
 	struct kdatacfg cfgdata;
@@ -630,10 +696,20 @@ gboolean on_DrawingPlot_draw(GtkWidget *widget, cairo_t *cr, gpointer data) {
 		pldata *plot = plot_data;
 		d1 = ref_d = mean_d = NULL;
 
+		color = (com.pref.combo_theme == 0) ? 0.0 : 1.0;
+
 		kplotcfg_defaults(&cfgplot);
 		kdatacfg_defaults(&cfgdata);
 		set_colors(&cfgplot);
+		cfgplot.ticlabel = TICLABEL_LEFT | TICLABEL_BOTTOM;
+		cfgplot.border = BORDER_ALL;
+		cfgplot.borderline.clr.type = KPLOTCTYPE_RGBA;
+		cfgplot.borderline.clr.rgba[0] = 0.5;
+		cfgplot.borderline.clr.rgba[1] = 0.5;
+		cfgplot.borderline.clr.rgba[2] = 0.5;
+		cfgplot.borderline.clr.rgba[3] = 1.0;
 		cfgplot.xaxislabel = xlabel == NULL ? _("Frames") : xlabel;
+		cfgplot.xtics = 3;
 		cfgplot.yaxislabel = ylabel;
 		cfgplot.yaxislabelrot = M_PI_2 * 3.0;
 		cfgplot.xticlabelpad = cfgplot.yticlabelpad = 10.0;
@@ -654,21 +730,19 @@ gboolean on_DrawingPlot_draw(GtkWidget *widget, cairo_t *cr, gpointer data) {
 		/* mean and min/max */
 		mean = kdata_ymean(d1);
 		//sigma = kdata_ystddev(d1);
-		min = plot_data->data[0].x;
-		/* if reference is ploted, we take it as maximum if it is */
-		max = (plot_data->data[plot_data->nb - 1].x > ref.x) ?
-				plot_data->data[plot_data->nb - 1].x + 1 : ref.x + 1;
+		int min_data = kdata_xmin(d1, NULL);
+		int max_data = kdata_xmax(d1, NULL);
 
 		if (nb_graphs == 1) {
-			avg = calloc(max - min, sizeof(struct kpair));
-			j = min;
-			for (i = 0; i < max - min; i++) {
+			avg = calloc((max_data - min_data) + 1, sizeof(struct kpair));
+			j = min_data;
+			for (i = 0; i < (max_data - min_data) + 1; i++) {
 				avg[i].x = plot_data->data[j].x;
 				avg[i].y = mean;
 				++j;
 			}
 
-			mean_d = kdata_array_alloc(avg, max - min);
+			mean_d = kdata_array_alloc(avg, (max_data - min_data) + 1);
 			kplot_attach_data(p, mean_d, KPLOT_LINES, NULL);	// mean plot
 			free(avg);
 
@@ -681,7 +755,7 @@ gboolean on_DrawingPlot_draw(GtkWidget *widget, cairo_t *cr, gpointer data) {
 		width = gtk_widget_get_allocated_width(widget);
 		height = gtk_widget_get_allocated_height(widget);
 
-		cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+		cairo_set_source_rgb(cr, color, color, color);
 		cairo_rectangle(cr, 0.0, 0.0, width, height);
 		cairo_fill(cr);
 		kplot_draw(p, width, height, cr);
@@ -697,6 +771,7 @@ gboolean on_DrawingPlot_draw(GtkWidget *widget, cairo_t *cr, gpointer data) {
 			requires_color_update = FALSE;
 		}
 
+		free_colors(&cfgplot);
 		kplot_free(p);
 		kdata_destroy(d1);
 		kdata_destroy(ref_d);
@@ -710,15 +785,32 @@ void on_plotCombo_changed(GtkComboBox *box, gpointer user_data) {
 	drawPlot();
 }
 
+void on_arcsecPhotometry_toggled(GtkToggleButton *button, gpointer user_data) {
+	is_arcsec = gtk_toggle_button_get_active(button);
+	drawPlot();
+}
+
+void on_JulianPhotometry_toggled(GtkToggleButton *button, gpointer user_data) {
+	force_Julian = gtk_toggle_button_get_active(button);
+	drawPlot();
+}
+
 static void update_ylabel() {
 	selected_source = gtk_combo_box_get_active(GTK_COMBO_BOX(combo));
 	gtk_widget_set_sensitive(varCurve, selected_source == MAGNITUDE);
+	gboolean arcsec_is_ok = (gfit.focal_length > 0.0 && gfit.pixel_size_x > 0.f
+			&& gfit.pixel_size_y > 0.f && gfit.binning_x > 0
+			&& gfit.binning_y > 0);
+	gtk_widget_set_visible(arcsec, selected_source == FWHM && arcsec_is_ok);
 	switch (selected_source) {
 	case ROUNDNESS:
 		ylabel = _("Star roundness (1 is round)");
 		break;
 	case FWHM:
-		ylabel = _("FWHM");
+		if (is_arcsec)
+			ylabel = _("FWHM ('')");
+		else
+			ylabel = _("FWHM (px)");
 		break;
 	case AMPLITUDE:
 		ylabel = _("Amplitude");
@@ -780,3 +872,131 @@ static void set_colors(struct kplotcfg *cfg) {
 	cfg->clrs[6].rgba[1] = 0x1e / 255.0;
 	cfg->clrs[6].rgba[2] = 0x10 / 255.0;
 }
+
+static void free_colors(struct kplotcfg *cfg) {
+	free(cfg->clrs);
+}
+
+static int get_index_of_frame(gdouble x, gdouble y) {
+	if (!plot_data) return -1;
+	if (!com.seq.imgparam) return -1;
+
+	pldata *plot = plot_data;
+
+	point min = { plot->frame[0], plot->data[0].y };
+	/* ensure to correctly find max frame index */
+	int i = 0;
+	while (plot->frame[com.seq.selnum - 1 - i] == 0.0) {
+		i++;
+	}
+	point max = { plot->frame[com.seq.selnum - 1 - i], plot->data[com.seq.selnum - 1 - i].y };
+	point intervale = { get_dimx() / (max.x - min.x), get_dimy() / (max.y - min.y)};
+	point pos = { x - get_offsx(), get_dimy() - y + get_offsy() };
+
+	int index = (int) round(pos.x / intervale.x) + (int) min.x - 1;
+	if (index >= 0 && index <= max.x && com.seq.imgparam[index].incl) {
+		return index;
+	}
+	return -1;
+}
+
+gboolean on_DrawingPlot_motion_notify_event(GtkWidget *widget,
+		GdkEventMotion *event, gpointer user_data) {
+
+	gtk_widget_set_has_tooltip(widget, FALSE);
+
+	int index = get_index_of_frame(event->x, event->y);
+	if (index >= 0) {
+		gchar *tooltip_text = g_strdup_printf("Frame: %d", (index + 1));
+		gtk_widget_set_tooltip_text(widget, tooltip_text);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+gboolean on_DrawingPlot_enter_notify_event(GtkWidget *widget, GdkEvent *event,
+		gpointer user_data) {
+	if (plot_data) {
+		set_cursor("tcross");
+	}
+	return TRUE;
+}
+
+gboolean on_DrawingPlot_leave_notify_event(GtkWidget *widget, GdkEvent *event,
+		gpointer user_data) {
+	if (get_thread_run()) {
+		set_cursor_waiting(TRUE);
+	} else {
+		/* trick to get default cursor */
+		set_cursor_waiting(FALSE);
+	}
+	return TRUE;
+}
+
+static void do_popup_plotmenu(GtkWidget *my_widget, GdkEventButton *event) {
+	static GtkMenu *menu = NULL;
+	static GtkMenuItem *menu_item = NULL;
+
+	int index = get_index_of_frame(event->x, event->y);
+	if (index < 0) return;
+
+	if (!menu) {
+		menu = GTK_MENU(lookup_widget("menu_plot"));
+		gtk_menu_attach_to_widget(GTK_MENU(menu), my_widget, NULL);
+		menu_item = GTK_MENU_ITEM(lookup_widget("menu_plot_exclude"));
+	}
+
+#if GTK_CHECK_VERSION(3, 22, 0)
+	gtk_menu_popup_at_pointer(GTK_MENU(menu), NULL);
+#else
+	int button, event_time;
+
+	if (event) {
+		button = event->button;
+		event_time = event->time;
+	} else {
+		button = 0;
+		event_time = gtk_get_current_event_time();
+	}
+
+	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button,
+			event_time);
+#endif
+	gchar *str = g_strdup_printf(_("Exclude Frame %d"), index + 1);
+	gtk_menu_item_set_label(menu_item, str);
+
+	g_free(str);
+}
+
+gboolean on_DrawingPlot_button_press_event(GtkWidget *widget,
+	GdkEventButton *event, gpointer user_data) {
+	do_popup_plotmenu(widget, event);
+	return TRUE;
+}
+
+static signed long extract_int_from_label(const gchar *str) {
+	gchar *p = (gchar *)str;
+	while (*p) {
+		if (g_ascii_isdigit(*p) || ((*p == '-' || *p == '+') && g_ascii_isdigit(*(p + 1)))) {
+	        // Found a number
+	        return g_ascii_strtoll(p, &p, 10); // return number
+	    } else {
+	        // Otherwise, move on to the next character.
+	        p++;
+	    }
+	}
+	return -1;
+}
+
+void on_menu_plot_exclude_activate(GtkMenuItem *menuitem, gpointer user_data) {
+	const gchar *label = gtk_menu_item_get_label(menuitem);
+	gint index;
+
+	index = extract_int_from_label(label);
+	if (index > 0) {
+		index--;
+
+		exclude_single_frame(index);
+		update_seqlist();
+	}
+}
