@@ -8,10 +8,16 @@ struct savedial_data {
 	gint quality;
 	const gchar *filename;
 	int bitpix;
+	gboolean update_hilo;
+	int retval;
 };
 
 enum {
 	PAGE_TIFF, PAGE_JPG, PAGE_FITS, PAGE_MISC
-} confirm;
+};
+
+void on_header_save_as_button_clicked();
+void on_header_snapshot_button_clicked();
+void on_header_save_button_clicked();
 
 #endif /* SRC_GUI_SAVE_DIALOG_H_ */
