@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -30,23 +30,23 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
-#include "gui/callbacks.h"
+#include "gui/utils.h"
 #include "gui/progress_and_log.h"
+#include "io/image_format_fits.h"
 
 #ifndef O_BINARY
 #define O_BINARY 0
 #endif
 
 static int bmp32tofits48(unsigned char *rvb, unsigned long rx, unsigned long ry, fits *fit) {
-	unsigned long datasize;
-	int i, j;
+	size_t datasize, i;
 	WORD *rdata, *gdata, *bdata, *olddata;
 
 	datasize = rx * ry;
 
 	olddata = fit->data;
 	if ((fit->data = realloc(fit->data, 3 * datasize * sizeof(WORD))) == NULL) {
-		printf("readbmp: could not alloc fit data\n");
+		PRINT_ALLOC_ERR;
 		if (olddata)
 			free(fit->data);
 		return 1;
@@ -55,13 +55,11 @@ static int bmp32tofits48(unsigned char *rvb, unsigned long rx, unsigned long ry,
 	rdata = fit->pdata[RLAYER] = fit->data;
 	gdata = fit->pdata[GLAYER] = fit->data + datasize;
 	bdata = fit->pdata[BLAYER] = fit->data + 2 * datasize;
-	for (i = 0; i < ry; i++) {
-		for (j = 0; j < rx; j++) {
-			*bdata++ = (WORD) *rvb++;
-			*gdata++ = (WORD) *rvb++;
-			*rdata++ = (WORD) *rvb++;
-			rvb++;
-		}
+	for (i = 0; i < datasize; i++) {
+		*bdata++ = (WORD) *rvb++;
+		*gdata++ = (WORD) *rvb++;
+		*rdata++ = (WORD) *rvb++;
+		rvb++;
 	}
 	fit->bitpix = fit->orig_bitpix = BYTE_IMG;
 	fit->naxis = 3;
@@ -79,11 +77,11 @@ static int bmp24tofits48(unsigned char *rvb, unsigned long rx, unsigned long ry,
 	WORD *rdata, *gdata, *bdata, *olddata;
 
 	int padsize = (4 - (rx * 3) % 4) % 4;
-	int newdatasize = ry * rx;
+	size_t newdatasize = ry * rx;
 
 	olddata = fit->data;
 	if ((fit->data = realloc(fit->data, 3 * newdatasize * sizeof(WORD))) == NULL) {
-		printf("readbmp: could not alloc fit data\n");
+		PRINT_ALLOC_ERR;
 		if (olddata)
 			free(fit->data);
 		return 1;
@@ -111,14 +109,12 @@ static int bmp24tofits48(unsigned char *rvb, unsigned long rx, unsigned long ry,
 }
 
 static int bmp16tofits48(unsigned char *rvb, unsigned long rx, unsigned long ry, fits *fit) {
-	int i;
 	WORD *rdata, *gdata, *bdata, *olddata;
-
-	int newdatasize = ry * rx;
+	size_t newdatasize = ry * rx;
 
 	olddata = fit->data;
 	if ((fit->data = realloc(fit->data, 3 * newdatasize * sizeof(WORD))) == NULL) {
-		printf("readbmp: could not alloc fit data\n");
+		PRINT_ALLOC_ERR;
 		if (olddata)
 			free(fit->data);
 		return 1;
@@ -126,15 +122,14 @@ static int bmp16tofits48(unsigned char *rvb, unsigned long rx, unsigned long ry,
 	rdata = fit->pdata[RLAYER] = fit->data;
 	gdata = fit->pdata[GLAYER] = fit->data + newdatasize;
 	bdata = fit->pdata[BLAYER] = fit->data + 2 * newdatasize;
-	for (i = 0; i < ry * rx; i++) {
+	for (size_t i = 0; i < newdatasize; i++) {
 		unsigned char buf0 = *rvb++;
 		unsigned char buf1 = *rvb++;
 		unsigned pixel_data = buf0 | buf1 << 8;
 
-        *rdata++ = ((pixel_data & 0x7c00) >> 10) * 255.0 / 31.0 + 0.5;
-        *gdata++ = ((pixel_data & 0x03e0) >> 5) * 255.0 / 31.0 + 0.5;
-        *bdata++ = ((pixel_data & 0x001f) >> 0) * 255.0 / 31.0 + 0.5;
-
+		*rdata++ = ((pixel_data & 0x7c00) >> 10) * 255.0 / 31.0 + 0.5;
+		*gdata++ = ((pixel_data & 0x03e0) >> 5) * 255.0 / 31.0 + 0.5;
+		*bdata++ = ((pixel_data & 0x001f) >> 0) * 255.0 / 31.0 + 0.5;
 	}
 	fit->bitpix = fit->orig_bitpix = BYTE_IMG;
 	fit->naxis = 3;
@@ -148,7 +143,7 @@ static int bmp16tofits48(unsigned char *rvb, unsigned long rx, unsigned long ry,
 }
 
 static int bmp8tofits(unsigned char *rgb, unsigned long rx, unsigned long ry, fits *fit) {
-	unsigned long nbdata, padsize;
+	size_t nbdata, padsize;
 	int i, j;
 	WORD *data, *olddata;
 
@@ -157,7 +152,7 @@ static int bmp8tofits(unsigned char *rgb, unsigned long rx, unsigned long ry, fi
 
 	olddata = fit->data;
 	if ((fit->data = realloc(fit->data, nbdata * sizeof(WORD))) == NULL) {
-		printf("readbmp: could not alloc fit data\n");
+		PRINT_ALLOC_ERR;
 		if (olddata)
 			free(fit->data);
 		return 1;
@@ -187,7 +182,6 @@ static void get_image_size(BYTE *header, unsigned long *width,
 	unsigned short sx = 0, sy = 0;
 
 	memcpy(&bitmapinfoheader, header + 14, 4);
-	printf("test: %lu\n", bitmapinfoheader);
 	if (bitmapinfoheader == 12) {
 		memcpy(&sx, header + 18, 2);
 		memcpy(&sy, header + 20, 2);
@@ -209,13 +203,10 @@ int readbmp(const char *name, fits *fit) {
 	unsigned char *buf;
 	unsigned long data_offset = 0;
 	unsigned long width = 0, height = 0;
-	unsigned long nbdata, padsize;
 	unsigned short nbplane = 0;
-	char *msg;
 
 	if ((file = g_fopen(name, "rb")) == NULL) {
-		msg = siril_log_message(_("Error opening BMP.\n"));
-		show_dialog(msg, _("Error"), "dialog-error-symbolic");
+		siril_log_message(_("Error opening BMP.\n"));
 		return -1;
 	}
 
@@ -226,51 +217,55 @@ int readbmp(const char *name, fits *fit) {
 		return -1;
 	}
 
-/*	memcpy(&compression, header + 30, 4);*/
+	/*	memcpy(&compression, header + 30, 4);*/
 
 	get_image_size(header, &width, &height);
 	memcpy(&nbplane, header + 28, 2);
 	nbplane = nbplane / 8;
 	memcpy(&data_offset, header + 10, 4);
 
-	padsize = (4 - (width * nbplane) % 4) % 4;
-	nbdata = width * height * nbplane + height * padsize;
+	unsigned int padsize = (4 - (width * nbplane) % 4) % 4;
+	size_t nbdata = width * height * nbplane + height * padsize;
 
-	fseek(file, data_offset, SEEK_SET);
-	if (nbplane == 1) {
-		buf = malloc(nbdata + 1024);
-		if ((count = fread(buf, 1, 1024, file)) != 1024) {
-			fprintf(stderr, "readbmp: %ld byte read instead of 1024\n", count);
-			perror("readbmp: failed to read the lut");
-			free(buf);
-			fclose(file);
-			return -1;
-		}
-	} else {
-		buf = malloc(nbdata);
+	if (fseek(file, data_offset, SEEK_SET) == -1) {
+		perror("BMP fseek for data");
+		fclose(file);
+		return -1;
 	}
 
-	fread(buf, 1, nbdata, file);
+	buf = malloc(nbdata);
+	if (!buf) {
+		PRINT_ALLOC_ERR;
+		fclose(file);
+		return -1;
+	}
+	unsigned long f;
+	if (nbdata != (f = fread(buf, 1, nbdata, file))) {
+		fprintf(stderr, "readbmp: could not read all data: (%zu, %lu)\n", nbdata, f);
+		free(buf);
+		fclose(file);
+		return -1;
+	}
 	fclose(file);
 
 	switch (nbplane) {
-	case 1:
-		bmp8tofits(buf, width, height, fit);
-		break;
-	case 2:
-		bmp16tofits48(buf, width, height, fit);
-		break;
-	case 3:
-		bmp24tofits48(buf, width, height, fit);
-		break;
-	case 4:
-		bmp32tofits48(buf, width, height, fit);
-		break;
-	default:
-		msg = siril_log_message(_("Sorry but Siril cannot "
-				"open this kind of BMP. Try to convert it before.\n"));
-		show_dialog(msg, _("Error"), "dialog-error-symbolic");
+		case 1:
+			bmp8tofits(buf, width, height, fit);
+			break;
+		case 2:
+			bmp16tofits48(buf, width, height, fit);
+			break;
+		case 3:
+			bmp24tofits48(buf, width, height, fit);
+			break;
+		case 4:
+			bmp32tofits48(buf, width, height, fit);
+			break;
+		default:
+			siril_log_message(_("Sorry but Siril cannot "
+						"open this kind of BMP. Try to convert it before.\n"));
 	}
+	fit->type = DATA_USHORT;
 	free(buf);
 	char *basename = g_path_get_basename(name);
 	siril_log_message(_("Reading BMP: file %s, %ld layer(s), %ux%u pixels\n"),
@@ -281,33 +276,35 @@ int readbmp(const char *name, fits *fit) {
 
 int savebmp(const char *name, fits *fit) {
 	unsigned char bmpfileheader[14] = { 'B', 'M', 	//Magic Number
-			0, 0, 0, 0, 	//Size in bytes, see below
-			0, 0, 0, 0, 54, 0, 0, 0	//offset
-			};
+		0, 0, 0, 0, 	//Size in bytes, see below
+		0, 0, 0, 0, 54, 0, 0, 0	//offset
+	};
 	unsigned char bmpinfoheader[40] = { 40, 0, 0, 0, //info of the header size
-			0, 0, 0, 0, 	//width, see below
-			0, 0, 0, 0, 	//height, see below
-			1, 0, 		//number color planes
-			24, 0,		//bits per pixel
-			0, 0, 0, 0, 	//no compression
-			0, 0, 0, 0, 	//image bits size
-			0, 0, 0, 0, 	//horizontal resolution, we don't care
-			0, 0, 0, 0, 	//vertical resolution, we don't care neither
-			0, 0, 0, 0, 	//colors in pallete
-			0, 0, 0, 0, 	//important colors
-			};
+		0, 0, 0, 0, 	//width, see below
+		0, 0, 0, 0, 	//height, see below
+		1, 0, 		//number color planes
+		24, 0,		//bits per pixel
+		0, 0, 0, 0, 	//no compression
+		0, 0, 0, 0, 	//image bits size
+		0, 0, 0, 0, 	//horizontal resolution, we don't care
+		0, 0, 0, 0, 	//vertical resolution, we don't care neither
+		0, 0, 0, 0, 	//colors in pallete
+		0, 0, 0, 0, 	//important colors
+	};
 	unsigned int width = fit->rx, height = fit->ry;
 	double norm;
 
 	FILE *f;
 
 	WORD *gbuf[3] = { fit->pdata[RLAYER], fit->pdata[GLAYER], fit->pdata[BLAYER] };
+	float *gbuff[3] = { fit->fpdata[RLAYER], fit->fpdata[GLAYER], fit->fpdata[BLAYER] };
 
-	int padsize = (4 - (width * 3) % 4) % 4;
-	int datasize = width * height * 3 + padsize * height;
-	int filesize = datasize + sizeof(bmpfileheader) + sizeof(bmpinfoheader);
+	unsigned int padsize = (4 - (width * 3) % 4) % 4;
+	size_t datasize = width * height * 3 + padsize * height;
+	size_t filesize = datasize + sizeof(bmpfileheader) + sizeof(bmpinfoheader);
 	int i, j;
 	WORD red, blue, green;
+	float redf, bluef, greenf;
 	unsigned char pixel[3];
 
 	bmpfileheader[2] = (unsigned char) (filesize);
@@ -331,43 +328,65 @@ int savebmp(const char *name, fits *fit) {
 	bmpinfoheader[27] = (unsigned char) (datasize >> 24);
 
 	char *filename = strdup(name);
-	if (!ends_with(filename, ".bmp")) {
+	if (!g_str_has_suffix(filename, ".bmp")) {
 		filename = str_append(&filename, ".bmp");
 	}
 
 	f = g_fopen(filename, "wb");
 	if (f == NULL) {
-		char *msg = siril_log_message(_("Can't create BMP file.\n"));
-		show_dialog(msg, _("Error"), "dialog-error-symbolic");
+		siril_log_message(_("Can't create BMP file.\n"));
 		free(filename);
 		return 1;
 	}
 
-	norm = (fit->orig_bitpix > BYTE_IMG ?
+	norm = (fit->orig_bitpix != BYTE_IMG ?
 			UCHAR_MAX_DOUBLE / USHRT_MAX_DOUBLE : 1.0);
 
 	fwrite(bmpfileheader, sizeof(bmpfileheader), 1, f);
 	fwrite(bmpinfoheader, sizeof(bmpinfoheader), 1, f);
 
-	for (i = 0; i < height; i++) {
-		for (j = 0; j < width; j++) {
-			red = *gbuf[RLAYER]++;
-			if (fit->naxes[2] == 3) {
-				green = *gbuf[GLAYER]++;
-				blue = *gbuf[BLAYER]++;
-			} else {
-				green = red;
-				blue = red;
+	if (fit->type == DATA_USHORT) {
+		for (i = 0; i < height; i++) {
+			for (j = 0; j < width; j++) {
+				red = *gbuf[RLAYER]++;
+				if (fit->naxes[2] == 3) {
+					green = *gbuf[GLAYER]++;
+					blue = *gbuf[BLAYER]++;
+				} else {
+					green = red;
+					blue = red;
+				}
+
+				pixel[0] = round_to_BYTE(blue * norm); /* swap Blue and Red */
+				pixel[1] = round_to_BYTE(green * norm);
+				pixel[2] = round_to_BYTE(red * norm);
+
+				fwrite(pixel, sizeof(pixel), 1, f);
 			}
+			if (padsize != 0)
+				fwrite("0", 1, padsize, f);		//We fill the end of width with 0
+		}
+	} else {
+		for (i = 0; i < height; i++) {
+			for (j = 0; j < width; j++) {
+				redf = *gbuff[RLAYER]++;
+				if (fit->naxes[2] == 3) {
+					greenf = *gbuff[GLAYER]++;
+					bluef = *gbuff[BLAYER]++;
+				} else {
+					greenf = redf;
+					bluef = redf;
+				}
 
-			pixel[0] = round_to_BYTE(blue * norm); /* swap Blue and Red */
-			pixel[1] = round_to_BYTE(green * norm);
-			pixel[2] = round_to_BYTE(red * norm);
+				pixel[0] = float_to_uchar_range(bluef); /* swap Blue and Red */
+				pixel[1] = float_to_uchar_range(greenf);
+				pixel[2] = float_to_uchar_range(redf);
 
-			fwrite(pixel, sizeof(pixel), 1, f);
+				fwrite(pixel, sizeof(pixel), 1, f);
+			}
+			if (padsize != 0)
+				fwrite("0", 1, padsize, f);	//We fill the end of width with 0
 		}
-		if (padsize != 0)
-			fwrite("0", 1, padsize, f);		//We fill the end of width with 0
 	}
 	fclose(f);
 	siril_log_message(_("Saving BMP: file %s, %ld layer(s), %ux%u pixels\n"), filename,
@@ -387,13 +406,13 @@ int savebmp(const char *name, fits *fit) {
 /* This method loads a pnm or pgm binary file into the fits image passed as argument. */
 int import_pnm_to_fits(const char *filename, fits *fit) {
 	FILE *file;
-	char buf[256], *msg;
-	int i, j, max_val;
+	char buf[256];
+	size_t i, j;
+	int max_val;
 	size_t stride;
 
 	if ((file = g_fopen(filename, "rb")) == NULL) {
-		msg = siril_log_message(_("Sorry but Siril cannot open this file.\n"));
-		show_dialog(msg, _("Error"), "dialog-error-symbolic");
+		siril_log_message(_("Sorry but Siril cannot open this file.\n"));
 		return -1;
 	}
 	if (fgets(buf, 256, file) == NULL) {
@@ -402,10 +421,9 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 		return -1;
 	}
 	if (buf[0] != 'P' || buf[1] < '5' || buf[1] > '6' || buf[2] != '\n') {
-		msg = siril_log_message(
+		siril_log_message(
 				_("Wrong magic cookie in PNM file, ASCII types and"
-						" b&w bitmaps are not supported.\n"));
-		show_dialog(msg, _("Error"), "dialog-error-symbolic");
+					" b&w bitmaps are not supported.\n"));
 		fclose(file);
 		return -1;
 	}
@@ -431,7 +449,7 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 		return -1;
 	}
 	buf[i] = '\0';
-	fit->rx = atoi(buf);
+	fit->rx = g_ascii_strtoull(buf, NULL, 10);
 	j = ++i;
 	while (buf[j] >= '0' && buf[j] <= '9')
 		j++;
@@ -444,7 +462,7 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 		return -1;
 	}
 	buf[j] = '\0';
-	fit->ry = atoi(buf + i);
+	fit->ry = g_ascii_strtoull(buf + i, NULL, 10);
 
 	do {
 		if (fgets(buf, 256, file) == NULL) {
@@ -460,7 +478,7 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 		return -1;
 	}
 	buf[i] = '\0';
-	max_val = atoi(buf);
+	max_val = g_ascii_strtoll(buf, NULL, 10);
 	if (max_val < UCHAR_MAX) {
 		fclose(file);
 		return -1;
@@ -477,7 +495,7 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 		olddata = fit->data;
 		fit->data = realloc(fit->data, stride * fit->ry * sizeof(WORD));
 		if (fit->data == NULL || tmpbuf == NULL) {
-			fprintf(stderr, "error allocating fits image data\n");
+			PRINT_ALLOC_ERR;
 			fclose(file);
 			if (olddata && !fit->data)
 				free(olddata);
@@ -487,8 +505,7 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 			return -1;
 		}
 		if (fread(tmpbuf, stride, fit->ry, file) < fit->ry) {
-			msg = siril_log_message(_("Error reading 8-bit PPM image data.\n"));
-			show_dialog(msg, _("Error"), "dialog-error-symbolic");
+			siril_log_message(_("Error reading 8-bit PPM image data.\n"));
 			fclose(file);
 			free(tmpbuf);
 			free(fit->data);
@@ -506,30 +523,28 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 	} else if (max_val == USHRT_MAX || max_val == SHRT_MAX) {
 		/* 16-bit file */
 		if (fit->naxes[2] == 1) {
-			int nbdata;
 			WORD *olddata = fit->data;
 			stride = fit->rx * sizeof(WORD);
 			fit->data = realloc(fit->data, stride * fit->ry * sizeof(WORD));
 			if (fit->data == NULL) {
-				fprintf(stderr, "error allocating fits image data\n");
+				PRINT_ALLOC_ERR;
 				fclose(file);
 				if (olddata)
 					free(olddata);
 				return -1;
 			}
 			if (fread(fit->data, stride, fit->ry, file) < fit->ry) {
-				msg = siril_log_message(
+				siril_log_message(
 						_("Error reading 16-bit gray PPM image data.\n"));
-				show_dialog(msg, _("Error"), "dialog-error-symbolic");
 				fclose(file);
 				free(fit->data);
 				fit->data = NULL;
 				return -1;
 			}
 			/* change endianness in place */
-			nbdata = fit->rx * fit->ry;
+			size_t nbdata = fit->rx * fit->ry;
 			for (i = 0; i < nbdata; i++)
-				fit->data[i] = (fit->data[i] >> 8) | (fit->data[i] << 8);
+				fit->data[i] = change_endianness16(fit->data[i]);
 			fit->pdata[0] = fit->data;
 			fit->pdata[1] = fit->data;
 			fit->pdata[2] = fit->data;
@@ -541,7 +556,7 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 			tmpbuf = malloc(stride * fit->ry);
 			fit->data = realloc(fit->data, stride * fit->ry * sizeof(WORD));
 			if (fit->data == NULL || tmpbuf == NULL) {
-				fprintf(stderr, "error allocating fits image data\n");
+				PRINT_ALLOC_ERR;
 				fclose(file);
 				if (olddata && !fit->data)
 					free(olddata);
@@ -551,9 +566,8 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 				return -1;
 			}
 			if (fread(tmpbuf, stride, fit->ry, file) < fit->ry) {
-				msg = siril_log_message(
+				siril_log_message(
 						_("Error reading 16-bit color PPM image data.\n"));
-				show_dialog(msg, _("Error"), "dialog-error-symbolic");
 				fclose(file);
 				free(tmpbuf);
 				free(fit->data);
@@ -567,12 +581,14 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 		fit->binning_x = fit->binning_y = 1;
 		fits_flip_top_to_bottom(fit);
 	} else {
-		msg = siril_log_message(_("Not handled max value for PNM: %d.\n"),
+		siril_log_message(_("Not handled max value for PNM: %d.\n"),
 				max_val);
-		show_dialog(msg, _("Error"), "dialog-error-symbolic");
 		fclose(file);
 		return -1;
 	}
+	fit->type = DATA_USHORT;
+	g_snprintf(fit->row_order, FLEN_VALUE, "%s", "TOP-DOWN");
+
 	fclose(file);
 	char *basename = g_path_get_basename(filename);
 	siril_log_message(_("Reading NetPBM: file %s, %ld layer(s), %ux%u pixels\n"),
@@ -583,32 +599,39 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 
 static int saveppm(const char *name, fits *fit) {
 	FILE *fp = g_fopen(name, "wb");
-	int i;
-	int ndata = fit->rx * fit->ry;
+	size_t i, ndata = fit->rx * fit->ry;
 	double norm;
 	const char *comment = "# CREATOR : SIRIL";
 
 	fprintf(fp, "P6\n%s\n%u %u\n%u\n", comment, fit->rx, fit->ry, USHRT_MAX);
-	WORD *gbuf[3] =
-			{ fit->pdata[RLAYER], fit->pdata[GLAYER], fit->pdata[BLAYER] };
+	WORD *gbuf[3] = { fit->pdata[RLAYER], fit->pdata[GLAYER], fit->pdata[BLAYER] };
+	float *gbuff[3] = { fit->fpdata[RLAYER], fit->fpdata[GLAYER], fit->fpdata[BLAYER] };
 	fits_flip_top_to_bottom(fit);
-	norm = (fit->orig_bitpix > BYTE_IMG) ? 1.0 : USHRT_MAX_DOUBLE / UCHAR_MAX_DOUBLE;
-	for (i = 0; i < ndata; i++) {
-		WORD color[3];
-		color[0] = *gbuf[RLAYER]++ * norm;
-		color[1] = *gbuf[GLAYER]++ * norm;
-		color[2] = *gbuf[BLAYER]++ * norm;
+	norm = (fit->orig_bitpix != BYTE_IMG) ? 1.0 : USHRT_MAX_DOUBLE / UCHAR_MAX_DOUBLE;
+	if (fit->type == DATA_USHORT) {
+		for (i = 0; i < ndata; i++) {
+			WORD color[3];
+			color[0] = *gbuf[RLAYER]++ * norm;
+			color[1] = *gbuf[GLAYER]++ * norm;
+			color[2] = *gbuf[BLAYER]++ * norm;
 
-		/* change endianness in place */
-		/* FIX ME : For a small amount of files (for example,
-		 * jpg converted to fit with iris),
-		 * this swap is not required and causes bad image 
-		 * THIS CASE SHOULD NOT BE VERY FREQUENT */
+			color[0] = change_endianness16(color[0]);
+			color[1] = change_endianness16(color[1]);
+			color[2] = change_endianness16(color[2]);
+			fwrite(color, sizeof(WORD), 3, fp);
+		}
+	} else {
+		for (i = 0; i < ndata; i++) {
+			WORD color[3];
+			color[0] = float_to_ushort_range(*gbuff[RLAYER]++);
+			color[1] = float_to_ushort_range(*gbuff[GLAYER]++);
+			color[2] = float_to_ushort_range(*gbuff[BLAYER]++);
 
-		color[0] = (color[0] >> 8) | (color[0] << 8);
-		color[1] = (color[1] >> 8) | (color[1] << 8);
-		color[2] = (color[2] >> 8) | (color[2] << 8);
-		fwrite(color, sizeof(WORD), 3, fp);
+			color[0] = change_endianness16(color[0]);
+			color[1] = change_endianness16(color[1]);
+			color[2] = change_endianness16(color[2]);
+			fwrite(color, sizeof(WORD), 3, fp);
+		}
 	}
 	fclose(fp);
 	fits_flip_top_to_bottom(fit);
@@ -619,10 +642,10 @@ static int saveppm(const char *name, fits *fit) {
 
 static int savepgm(const char *name, fits *fit) {
 	FILE *fp;
-	int i;
-	int ndata = fit->rx * fit->ry;
+	size_t i, ndata = fit->rx * fit->ry;
 	double norm;
 	WORD *gbuf = fit->pdata[RLAYER];
+	float *gbuff = fit->fpdata[RLAYER];
 	const char *comment = "# CREATOR : SIRIL";
 
 	fp = g_fopen(name, "wb");
@@ -631,13 +654,23 @@ static int savepgm(const char *name, fits *fit) {
 	fprintf(fp, "P5\n%s\n%u %u\n%u\n", comment, fit->rx, fit->ry, USHRT_MAX);
 
 	fits_flip_top_to_bottom(fit);
-	norm = (fit->orig_bitpix > BYTE_IMG) ? 1.0 : USHRT_MAX_DOUBLE / UCHAR_MAX_DOUBLE;
-	for (i = 0; i < ndata; i++) {
-		WORD tmp = *gbuf++ * norm;
-		/* change endianness in place */
-		WORD data[1];
-		data[0] = (tmp >> 8) | (tmp << 8);
-		fwrite(data, sizeof(data), 1, fp);
+	norm = (fit->orig_bitpix != BYTE_IMG) ? 1.0 : USHRT_MAX_DOUBLE / UCHAR_MAX_DOUBLE;
+	if (fit->type == DATA_USHORT) {
+		for (i = 0; i < ndata; i++) {
+			WORD tmp = *gbuf++ * norm;
+			/* change endianness in place */
+			WORD data[1];
+			data[0] = (tmp >> 8) | (tmp << 8);
+			fwrite(data, sizeof(data), 1, fp);
+		}
+	} else {
+		for (i = 0; i < ndata; i++) {
+			WORD tmp = float_to_ushort_range(*gbuff++);
+			/* change endianness in place */
+			WORD data[1];
+			data[0] = (tmp >> 8) | (tmp << 8);
+			fwrite(data, sizeof(data), 1, fp);
+		}
 	}
 	fclose(fp);
 	fits_flip_top_to_bottom(fit);
@@ -651,12 +684,12 @@ int saveNetPBM(const char *name, fits *fit) {
 	char *filename = strdup(name);
 
 	if (fit->naxes[2] == 1) {
-		if (!ends_with(filename, ".pgm")) {
+		if (!g_str_has_suffix(filename, ".pgm")) {
 			filename = str_append(&filename, ".pgm");
 		}
 		retval = savepgm(filename, fit);
 	} else {
-		if (!ends_with(filename, ".ppm") && !ends_with(filename, ".pnm")) {
+		if (!g_str_has_suffix(filename, ".ppm") && !g_str_has_suffix(filename, ".pnm")) {
 			filename = str_append(&filename, ".ppm");
 		}
 		retval = saveppm(filename, fit);
@@ -667,13 +700,11 @@ int saveNetPBM(const char *name, fits *fit) {
 
 
 static int pictofit(WORD *buf, fits *fit) {
-	int nbdata;
-	int i;
 	WORD *data, *olddata = fit->data;
 
-	nbdata = fit->rx * fit->ry;
+	size_t i, nbdata = fit->rx * fit->ry;
 	if ((fit->data = realloc(fit->data, nbdata * sizeof(WORD))) == NULL) {
-		fprintf(stderr, "readpic: could not alloc fit data\n");
+		PRINT_ALLOC_ERR;
 		if (olddata)
 			free(olddata);
 		return -1;
@@ -691,16 +722,15 @@ static int pictofit(WORD *buf, fits *fit) {
 	fit->naxes[1] = fit->ry;
 	fit->naxes[2] = 1;
 	fit->naxis = 2;
-	return 1;
+	return 0;
 }
 
 static int pictofitrgb(WORD *buf, fits *fit) {
-	int i, nbdata;
 	WORD *data[3], *olddata = fit->data;
 
-	nbdata = fit->rx * fit->ry;
+	size_t i, nbdata = fit->rx * fit->ry;
 	if ((fit->data = realloc(fit->data, nbdata * 3 * sizeof(WORD))) == NULL) {
-		fprintf(stderr, "readpic: could not alloc fit data\n");
+		PRINT_ALLOC_ERR;
 		if (olddata)
 			free(olddata);
 		return -1;
@@ -731,7 +761,7 @@ static int pictofitrgb(WORD *buf, fits *fit) {
 	fit->naxes[0] = fit->rx;
 	fit->naxes[1] = fit->ry;
 	fit->naxes[2] = 3;
-	return 3;
+	return 0;
 }
 
 static int _pic_read_header(struct pic_struct *pic_file) {
@@ -746,9 +776,8 @@ static int _pic_read_header(struct pic_struct *pic_file) {
 	memcpy(&pic_file->magic, header, 4);
 
 	if (pic_file->magic != 0x12231fc) {
-		char *msg = siril_log_message(_("Wrong magic cookie in PIC file. "
-				"This image is not supported.\n"));
-		show_dialog(msg, _("Error"), "dialog-error-symbolic");
+		siril_log_message(_("Wrong magic cookie in PIC file. "
+					"This image is not supported.\n"));
 		return -1;
 	}
 
@@ -773,35 +802,30 @@ static int _pic_close_file(struct pic_struct *pic_file) {
 		retval = fclose(pic_file->file);
 		pic_file->file = NULL;
 	}
-	if (pic_file->date)
-		free(pic_file->date);
-	if (pic_file->time)
-		free(pic_file->time);
-	memset(pic_file, 0, sizeof(struct pic_struct));
+	g_free(pic_file->date);
+	g_free(pic_file->time);
 	free(pic_file);
 	return retval;
 }
 
 int readpic(const char *name, fits *fit) {
-	char header[290];
 	struct pic_struct *pic_file;
 	WORD *buf;
-	char *msg;
 	int retval = 0;
-	unsigned int nbdata;
 
-	memset(&header, 0, sizeof(header));
 	pic_file = calloc(1, sizeof(struct pic_struct));
 
 	if ((pic_file->file = g_fopen(name, "rb")) == NULL) {
-		msg = siril_log_message(
+		siril_log_message(
 				_("Sorry but Siril cannot open the PIC file: %s.\n"), name);
-		show_dialog(msg, _("Error"), "dialog-error-symbolic");
 		free(pic_file);
 		return -1;
 	}
 
-	_pic_read_header(pic_file);
+	if (_pic_read_header(pic_file)) {
+		_pic_close_file(pic_file);
+		return -1;
+	}
 
 	fit->rx = (unsigned int) pic_file->width;
 	fit->ry = (unsigned int) pic_file->height;
@@ -809,10 +833,11 @@ int readpic(const char *name, fits *fit) {
 	fit->binning_y = (unsigned int) pic_file->bin[5];
 	fit->hi = pic_file->hi;
 	fit->lo = pic_file->lo;
+	fit->type = DATA_USHORT;
 
-	nbdata = fit->rx * fit->ry;
-	assert(nbdata > 0);
-	fseek(pic_file->file, sizeof(header), SEEK_SET);
+	size_t nbdata = fit->rx * fit->ry;
+
+	fseek(pic_file->file, 290, SEEK_SET);
 	buf = malloc(nbdata * pic_file->nbplane * sizeof(WORD));
 
 	if ((fread(buf, 1, nbdata * pic_file->nbplane * sizeof(WORD), pic_file->file))
@@ -824,22 +849,26 @@ int readpic(const char *name, fits *fit) {
 	}
 
 	switch (pic_file->nbplane) {
-	case 1:
-		retval = pictofit(buf, fit);
-		break;
-	case 3:
-		retval = pictofitrgb(buf, fit);
-		break;
-	default:
-		retval = -1;
-		msg = siril_log_message(_("Sorry but Siril cannot open this file.\n"));
-		show_dialog(msg, _("Error"), "dialog-error-symbolic");
+		case 1:
+			retval = pictofit(buf, fit);
+			break;
+		case 3:
+			retval = pictofitrgb(buf, fit);
+			break;
+		default:
+			retval = -1;
+			siril_log_message(_("Sorry but Siril cannot open this file.\n"));
+	}
+	free(buf);
+
+	if (retval) {
+		_pic_close_file(pic_file);
+		return -1;
 	}
 
 	char *basename = g_path_get_basename(name);
 	siril_log_message(_("Reading PIC: file %s, %ld layer(s), %ux%u pixels\n"),
 			basename, fit->naxes[2], fit->rx, fit->ry);
-	g_free(basename);
 	siril_log_message("(%d,%d)-(%d,%d) - Binning %dx%d\n", pic_file->bin[0],
 			pic_file->bin[1], pic_file->bin[2], pic_file->bin[3],
 			fit->binning_x, fit->binning_y);
@@ -854,6 +883,6 @@ int readpic(const char *name, fits *fit) {
 	}
 
 	_pic_close_file(pic_file);
-	free(buf);
+	g_free(basename);
 	return retval;
 }
