@@ -1,11 +1,16 @@
 #ifndef _SER_H_
 #define _SER_H_
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
-#include <stdint.h>
 #ifdef _OPENMP
 #include <omp.h>
 #endif
+#include "core/siril.h"
+#include "io/seqwriter.h"
 
 /* This file is part of Siril, https://free-astro.org/
  *
@@ -13,14 +18,14 @@
  * on big endian systems.
  */
 
-#if defined(__unix__) || (defined(__APPLE__) && defined(__MACH__))
+#if defined(__unix__) || (defined(OS_OSX))
 #include <sys/param.h>		// define or not BSD macro
 #endif
 
 #ifdef __linux__
 #define fseek64 fseeko  // Linux
 #define ftell64 ftello  // Linux
-#elif defined (__APPLE__) && defined(__MACH__)
+#elif defined (OS_OSX)
 #define fseek64 fseeko  // OS X
 #define ftell64 ftello  // OS X
 #elif defined(BSD)
@@ -32,6 +37,9 @@
 #elif defined (__gnu_hurd__)
 #define fseek64 fseeko  // GNU/Hurd
 #define ftell64 ftello  // GNU/Hurd
+#elif defined(__CYGWIN__)
+#define fseek64 fseeko  // CYGWIN
+#define ftell64 ftello  // CYGWIN
 #else
 #define fseek64 _fseeki64  // Windows
 #define ftell64 _ftelli64  // Windows
@@ -90,16 +98,16 @@ struct ser_struct {
 	char observer[40];		// 40	(42)
 	char instrument[40];		// 40	(82)
 	char telescope[40];		// 40	(122)
-	uint64_t date;		// 8	(162)
-	uint64_t date_utc;	// 8 (170)
+	guint64 date;		// 8	(162)
+	guint64 date_utc;	// 8 (170)
 
 	/* timestamps (not in the header, timestamps are in trailer) */
-	uint64_t *ts;			// total timestamps
+	guint64 *ts;			// total timestamps
 	int ts_alloc;			// allocated number of elements in ts
-	uint64_t ts_min, ts_max;// min and max timestamp
+	guint64 ts_min, ts_max;// min and max timestamp
 	double fps;				// frame rate
 
-	int64_t filesize;			// size of the file
+	gint64 filesize;			// size of the file
 
 	// internal representations of header data
 	ser_pixdepth byte_pixel_depth;	// more useful representation of the bit_pixel_depth
@@ -109,6 +117,8 @@ struct ser_struct {
 #ifdef _OPENMP
 	omp_lock_t fd_lock, ts_lock;
 #endif
+
+	struct seqwriter_data *writer;
 };
 
 gboolean ser_is_cfa(struct ser_struct *ser_file);
@@ -116,16 +126,21 @@ void ser_convertTimeStamp(struct ser_struct *ser_file, GSList *timestamp);
 void ser_init_struct(struct ser_struct *ser_file);
 void ser_display_info(struct ser_struct *ser_file);
 int ser_open_file(const char *filename, struct ser_struct *ser_file);
+int ser_close_and_delete_file(struct ser_struct *ser_file);
 int ser_write_and_close(struct ser_struct *ser_file);
-int ser_compact_file(struct ser_struct *ser_file, unsigned char *successful_frames, int nb_frames);
 int ser_create_file(const char *filename, struct ser_struct *ser_file, gboolean overwrite, struct ser_struct *copy_from);
 int ser_close_file(struct ser_struct *ser_file);
-int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit);
+int ser_metadata_as_fits(struct ser_struct *ser_file, fits *fit);
+int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit, gboolean force_float);
 int ser_read_opened_partial_fits(struct ser_struct *ser_file, int layer,
 		int frame_no, fits *fit, const rectangle *area);
 int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 		int frame_no, WORD *buffer, const rectangle *area);
 int ser_write_frame_from_fit(struct ser_struct *ser_file, fits *fit, int frame);
+gint64 ser_compute_file_size(struct ser_struct *ser_file, int nb_frames);
+int import_metadata_from_serfile(struct ser_struct *ser_file, fits *to);
+GdkPixbuf* get_thumbnail_from_ser(char *filename, gchar **descr);
+GDateTime *ser_read_frame_date(struct ser_struct *ser_file, int frame_no);
 
 #endif
 
