@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -24,11 +24,21 @@
 #include <string.h>
 
 #include "core/siril.h"
-#include "core/proto.h"
+#include "core/OS_utils.h"
 #include "algos/statistics.h"
+#include "algos/annotate.h"
+#include "algos/background_extraction.h"
+#include "algos/plateSolver.h"
+#include "gui/image_interactions.h"
+#include "gui/image_display.h"
+#include "gui/utils.h"
 #include "gui/callbacks.h"
+#include "gui/dialogs.h"
+#include "gui/message_dialog.h"
+#include "gui/preferences.h"
 #include "io/conversion.h"
 #include "io/sequence.h"
+#include "io/image_format_fits.h"
 #include "io/single_image.h"
 #include "gui/PSF_list.h"
 #include "gui/histogram.h"
@@ -43,6 +53,12 @@ void close_single_image() {
 	if (sequence_is_loaded() && com.seq.current >= 0)
 		return;
 	fprintf(stdout, "MODE: closing single image\n");
+	/* we need to close all dialogs in order to avoid bugs
+	 * with previews
+	 */
+	if (!com.headless) {
+		siril_close_preview_dialogs();
+	}
 	free_image_data();
 	undo_flush();
 }
@@ -50,21 +66,28 @@ void close_single_image() {
 /* frees resources when changing sequence or closing a single image
  * (image size may vary, so we reallocate) */
 void free_image_data() {
-	int i=0, vport;
+	int vport;
 	fprintf(stdout, "free_image_data() called, clearing loaded image\n");
 	/* WARNING: single_image.fit references the actual fits image,
 	 * shouldn't it be used here instead of gfit? */
 	if (!single_image_is_loaded() && sequence_is_loaded())
 		save_stats_from_fit(&gfit, &com.seq, com.seq.current);
 	clearfits(&gfit);
-	if (!com.script) {
+	invalidate_WCS_keywords(&gfit);
+	if (!com.headless) {
 		clear_stars_list();
 		delete_selected_area();
 		clear_sampling_setting_box();	// clear focal and pixel pitch info
+		free_background_sample_list(com.grad_samples);
+		com.grad_samples = NULL;
+		g_slist_free_full(com.found_object, (GDestroyNotify)free_object);
+		com.found_object = NULL;
+		reset_display_offset();
+		reset_zoom_default();
 	}
 	clear_histograms();
 
-	for (vport=0; vport<MAXGRAYVPORT; vport++) {
+	for (vport = 0; vport < MAXGRAYVPORT; vport++) {
 		if (com.graybuf[vport]) {
 			free(com.graybuf[vport]);
 			com.graybuf[vport] = NULL;
@@ -76,24 +99,25 @@ void free_image_data() {
 		com.surface_stride[vport] = 0;
 		com.surface_height[vport] = 0;
 	}
-	if (!com.script)
+	if (!com.headless)
 		activate_tab(RED_VPORT);
 	if (com.rgbbuf) {
 		free(com.rgbbuf);
 		com.rgbbuf = NULL;
 	}
 	if (com.uniq) {
-		if (com.uniq->filename)
-			free(com.uniq->filename);
-		if (com.uniq->comment)
-			free(com.uniq->comment);
+		free(com.uniq->filename);
+		com.uniq->fileexist = FALSE;
+		free(com.uniq->comment);
+		free(com.uniq->layers);
 		free(com.uniq);
 		com.uniq = NULL;
 	}
- 
-	if (!com.script) {
+
+	if (!com.headless) {
 		/* free alignment preview data */
-		for (i=0; i<PREVIEW_NB; i++) {
+		int i;
+		for (i = 0; i < PREVIEW_NB; i++) {
 			if (com.preview_surface[i]) {
 				cairo_surface_destroy(com.preview_surface[i]);
 				com.preview_surface[i] = NULL;
@@ -112,33 +136,51 @@ static gboolean end_read_single_image(gpointer p) {
 	return FALSE;
 }
 
-/* Reads an image from disk and stores it in the user allocated destination
- * fits. The realname_out argument can be NULL, and if not, it is set to the
+/**
+ * Reads an image from disk and stores it in the user allocated destination
+ * fits.
+ * @param filename
+ * @param dest
+ * @param realname_out argument can be NULL, and if not, it is set to the
  * real file name of the loaded file, since the given filename can be without
- * extension. This realname output has to be freed by user.
+ * extension.
+ * @param is_sequence is set to TRUE if the loaded image is in fact a SER or AVI sequence. Can be NULL
+ * @return
  */
-int read_single_image(const char* filename, fits *dest, char **realname_out) {
+int read_single_image(const char *filename, fits *dest, char **realname_out,
+		gboolean allow_sequences, gboolean *is_sequence, gboolean allow_dialogs,
+		gboolean force_float) {
 	int retval;
 	image_type imagetype;
 	char *realname = NULL;
+	gboolean single_sequence = FALSE;
 
 	retval = stat_file(filename, &imagetype, &realname);
 	if (retval) {
 		char *msg = siril_log_message(_("Error opening image %s: file not found or not supported.\n"), filename);
-		show_dialog(msg, _("Error"), "dialog-error-symbolic");
+		siril_message_dialog( GTK_MESSAGE_ERROR, _("Error"), msg);
 		set_cursor_waiting(FALSE);
 		free(realname);
 		return 1;
 	}
-	if (imagetype == TYPESER || imagetype == TYPEAVI) {
-			/* Returns 3 if ok */
-		retval = read_single_sequence(realname, imagetype);
+	if (imagetype == TYPESER || imagetype == TYPEAVI ||
+				(imagetype == TYPEFITS && fitseq_is_fitseq(realname, NULL))) {
+		if (allow_sequences) {
+			retval = read_single_sequence(realname, imagetype);
+			single_sequence = TRUE;
+		} else {
+			siril_log_message(_("Cannot open a sequence from here\n"));
+			return 1;
+		}
 	} else {
-		retval = any_to_fits(imagetype, realname, dest);
+		retval = any_to_fits(imagetype, realname, dest, allow_dialogs, force_float, com.pref.debayer.open_debayer);
 		if (!retval)
-			debayer_if_needed(imagetype, dest, com.debayer.compatibility, FALSE);
+			debayer_if_needed(imagetype, dest, FALSE);
 	}
-	if (retval != 0 && retval != 3)
+	if (is_sequence) {
+		*is_sequence = single_sequence;
+	}
+	if (retval && retval != OPEN_IMAGE_CANCEL)
 		siril_log_message(_("Opening %s failed.\n"), realname);
 	if (realname_out)
 		*realname_out = realname;
@@ -150,8 +192,7 @@ int read_single_image(const char* filename, fits *dest, char **realname_out) {
 }
 
 static gboolean end_open_single_image(gpointer arg) {
-	char *name = (char *)arg;
-	open_single_image_from_gfit(name);
+	open_single_image_from_gfit();
 	return FALSE;
 }
 
@@ -163,43 +204,45 @@ static gboolean end_open_single_image(gpointer arg) {
 int open_single_image(const char* filename) {
 	int retval;
 	char *realname;
+	gboolean is_single_sequence;
 
-	close_single_image();	// close the previous image and free resources
 	close_sequence(FALSE);	// closing a sequence if loaded
+	close_single_image();	// close the previous image and free resources
 
-	retval = read_single_image(filename, &gfit, &realname);
-	
-	/* A single sequence has been loaded */
-	if (retval == 3) {
-		return 0;
-	}
+	retval = read_single_image(filename, &gfit, &realname, TRUE, &is_single_sequence, TRUE, FALSE);
 	if (retval == 2) {
-		show_dialog(_("This file could not be opened because its extension is not supported.\n"), _("Error"), "dialog-error-symbolic");
-		return 1;
-	}
-	if (retval == 1) {
-		show_dialog(_("There was an error when opening this image. See the log for more information."), _("Error"), "dialog-error-symbolic");
+		siril_message_dialog(GTK_MESSAGE_ERROR, _("Error opening file"),
+				_("This file could not be opened because "
+						"its extension is not supported."));
 		return 1;
 	}
 
-	fprintf(stdout, "Loading image OK, now displaying\n");
+	if (retval < 0) {
+		siril_message_dialog(GTK_MESSAGE_ERROR, _("Error opening file"),
+				_("There was an error when opening this image. "
+						"See the log for more information."));
+		return 1;
+	}
 
-	/* Now initializing com struct */
-	com.seq.current = UNRELATED_IMAGE;
-	com.uniq = calloc(1, sizeof(single));
-	com.uniq->filename = realname;
-	com.uniq->nb_layers = gfit.naxes[2];
-	com.uniq->layers = calloc(com.uniq->nb_layers, sizeof(layer_info));
-	com.uniq->fit = &gfit;
-	siril_add_idle(end_open_single_image, realname);
-	return 0;
+	if (!is_single_sequence) {
+		fprintf(stdout, "Loading image OK, now displaying\n");
+
+		/* Now initializing com struct */
+		com.seq.current = UNRELATED_IMAGE;
+		com.uniq = calloc(1, sizeof(single));
+		com.uniq->filename = realname;
+		com.uniq->fileexist = get_type_from_filename(com.uniq->filename) == TYPEFITS;
+		com.uniq->nb_layers = gfit.naxes[2];
+		com.uniq->layers = calloc(com.uniq->nb_layers, sizeof(layer_info));
+		com.uniq->fit = &gfit;
+		siril_add_idle(end_open_single_image, realname);
+	}
+	return retval;
 }
 
 /* creates a single_image structure and displays a single image, found in gfit.
- * The argument, realname, should be the file path, and it must be allocated
- * on the heap, since destroying the single image will attempt to free it.
  */
-void open_single_image_from_gfit(char *realname) {
+void open_single_image_from_gfit() {
 	/* now initializing everything
 	 * code based on seq_load_image or set_seq (sequence.c) */
 
@@ -216,20 +259,14 @@ void open_single_image_from_gfit(char *realname) {
 	adjust_sellabel();
 
 	display_filename();	// display filename in gray window
+	set_precision_switch(); // set precision on screen
 
 	/* update menus */
 	update_MenuItem();
 
-	redraw(com.cvport, REMAP_ALL);
-	update_used_memory();
-	show_main_gray_window();
-	adjust_vport_size_to_image();
-	update_gfit_histogram_if_needed();
-	if (gfit.naxes[2] == 3)
-		show_rgb_window();
-	else
-		hide_rgb_window();
 	close_tab();
+	update_gfit_histogram_if_needed();
+	redraw(com.cvport, REMAP_ALL);
 }
 
 /* searches the image for minimum and maximum pixel value, on each layer
@@ -241,8 +278,8 @@ int image_find_minmax(fits *fit) {
 	for (layer = 0; layer < fit->naxes[2]; ++layer) {
 		// calling statistics() saves stats in the fit already, we don't need
 		// to use the returned handle
-		free_stats(statistics(NULL, -1, fit, layer, NULL, STATS_MINMAX));
-		if (!fit->stats[layer])
+		free_stats(statistics(NULL, -1, fit, layer, NULL, STATS_MINMAX, TRUE));
+		if (!fit->stats || !fit->stats[layer])
 			return -1;
 		fit->maxi = max(fit->maxi, fit->stats[layer]->max);
 		fit->mini = min(fit->mini, fit->stats[layer]->min);
@@ -253,7 +290,7 @@ int image_find_minmax(fits *fit) {
 static int fit_get_minmax(fits *fit, int layer) {
 	// calling statistics() saves stats in the fit already, we don't need
 	// to use the returned handle
-	free_stats(statistics(NULL, -1, fit, layer, NULL, STATS_MINMAX));
+	free_stats(statistics(NULL, -1, fit, layer, NULL, STATS_MINMAX, FALSE));
 	if (!fit->stats[layer])
 		return -1;
 	return 0;
@@ -271,6 +308,17 @@ double fit_get_min(fits *fit, int layer) {
 	return fit->stats[layer]->min;
 }
 
+static void fit_lohi_to_layers(fits *fit, double lo, double hi, layer_info *layer) {
+	if (fit->type == DATA_USHORT) {
+		layer->lo = (WORD)lo;
+		layer->hi = (WORD)hi;
+	}
+	else if (fit->type == DATA_FLOAT) {
+		layer->lo = float_to_ushort_range((float)lo);
+		layer->hi = float_to_ushort_range((float)hi);
+	}
+}
+
 /* gfit has been loaded, now we copy the hi and lo values into the com.uniq or com.seq layers.
  * gfit.hi and gfit.lo may only be available in some FITS files; if they are not available, the
  * min and max value for the layer is used.
@@ -301,13 +349,12 @@ void init_layers_hi_and_lo_values(sliders_mode force_minmax) {
 		if (gfit.hi == 0 || force_minmax == MINMAX) {
 			com.sliders = MINMAX;
 			if (!is_chained) {
-				layers[i].hi = fit_get_max(&gfit, i);
-				layers[i].lo = fit_get_min(&gfit, i);
+				fit_lohi_to_layers(&gfit, fit_get_min(&gfit, i),
+						fit_get_max(&gfit, i), &layers[i]);
 			}
 			else {
 				image_find_minmax(&gfit);
-				layers[i].hi = gfit.maxi;
-				layers[i].lo = gfit.mini;
+				fit_lohi_to_layers(&gfit, gfit.mini, gfit.maxi, &layers[i]);
 			}
 		} else {
 			com.sliders = MIPSLOHI;
@@ -321,35 +368,13 @@ void init_layers_hi_and_lo_values(sliders_mode force_minmax) {
 void adjust_cutoff_from_updated_gfit() {
 	invalidate_stats_from_fit(&gfit);
 	if (!com.script) {
-		update_gfit_histogram_if_needed();
+		invalidate_gfit_histogram();
+		compute_histo_for_gfit();
 		init_layers_hi_and_lo_values(com.sliders);
 		set_cutoff_sliders_values();
 	}
 }
 
-void unique_free_preprocessing_data(single *uniq) {
-	// free opened files
-	if (uniq->ppprefix) {
-		free(uniq->ppprefix);
-		uniq->ppprefix = NULL;
-	}
-	if (uniq->offset) {
-		clearfits(uniq->offset);
-		free(uniq->offset);
-		uniq->offset = NULL;
-	}
-	if (uniq->dark) {
-		clearfits(uniq->dark);
-		free(uniq->dark);
-		uniq->dark = NULL;
-	}
-	if (uniq->flat) {
-		clearfits(uniq->flat);
-		free(uniq->flat);
-		uniq->flat = NULL;
-	}
-}
-
 int single_image_is_loaded() {
 	return (com.uniq != NULL && com.uniq->nb_layers > 0);
 }
