@@ -16,16 +16,16 @@ extern "C" {
 
 
 int cvResizeGaussian(fits *, int, int, int);
-int cvResizeGaussian_data8(uint8_t *dataIn, int rx, int ry, uint8_t *dataOut,
+int cvResizeGaussian_uchar(uint8_t *dataIn, int rx, int ry, uint8_t *dataOut,
 		int toX, int toY, int chan, int interpolation);
-int cvRotateImage(fits *, double, int, int);
-int cvCalculH(s_star *star_array_img,
+int cvRotateImage(fits *, point, double, int, int);
+unsigned char *cvCalculH(s_star *star_array_img,
 		struct s_star *star_array_ref, int n, Homography *H);
 int cvApplyScaleToH(Homography *H1, double scale);
-int cvTransformImage(fits *, point, Homography, int);
+int cvTransformImage(fits *image, long width, long height, Homography Hom, int interpolation);
 int cvUnsharpFilter(fits*, double, double);
-int cvComputeFinestScale(fits *image);
 int cvLucyRichardson(fits *image, double sigma, int iterations);
+int cvClahe(fits *image, double clip_limit, int size);
 #ifdef __cplusplus
 }
 #endif
