@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -22,18 +22,22 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
-#include <assert.h>
-#include "registration/registration.h"
+
 #include "core/siril.h"
-#include "core/processing.h"
 #include "core/proto.h"
+#include "core/processing.h"
+#include "core/OS_utils.h"
 #include "algos/star_finder.h"
+#include "algos/statistics.h"
 #include "algos/PSF.h"
+#include "gui/image_display.h"
 #include "gui/PSF_list.h"
 #include "gui/progress_and_log.h"
-#include "gui/callbacks.h"
+#include "gui/utils.h"
 #include "io/sequence.h"
 #include "io/ser.h"
+#include "io/image_format_fits.h"
+#include "registration/registration.h"
 #include "registration/matching/atpmatch.h"
 #include "registration/matching/match.h"
 #include "registration/matching/misc.h"
@@ -47,23 +51,24 @@
  */
 
 static void create_output_sequence_for_global_star(struct registration_args *args);
-static void print_alignment_results(Homography H, int filenum, float FWHMx, float FWHMy);
+static void print_alignment_results(Homography H, int filenum, float FWHMx, float FWHMy, char *units);
 
 struct star_align_data {
 	struct registration_args *regargs;
 	regdata *current_regdata;
 	fitted_PSF **refstars;
 	int fitted_stars;
-	point ref;
 	BYTE *success;
+	point ref;
 };
 
 static int star_align_prepare_hook(struct generic_seq_args *args) {
 	struct star_align_data *sadata = args->user;
 	struct registration_args *regargs = sadata->regargs;
 	float FWHMx, FWHMy;
+	char *units;
 	fits fit = { 0 };
-	int i, nb_stars;
+	int i, nb_stars = 0;
 
 	if (args->seq->regparam[regargs->layer]) {
 		siril_log_message(
@@ -74,18 +79,20 @@ static int star_align_prepare_hook(struct generic_seq_args *args) {
 	} else {
 		sadata->current_regdata = calloc(args->seq->number, sizeof(regdata));
 		if (sadata->current_regdata == NULL) {
-			fprintf(stderr, "Error allocating registration data\n");
+			PRINT_ALLOC_ERR;
 			return -2;
 		}
+		args->seq->regparam[regargs->layer] = sadata->current_regdata;
 	}
 
 	/* first we're looking for stars in reference image */
-	if (seq_read_frame(args->seq, regargs->reference_image, &fit)) {
+	if (seq_read_frame(args->seq, regargs->reference_image, &fit, FALSE, -1)) {
 		siril_log_message(_("Could not load reference image\n"));
 		args->seq->regparam[regargs->layer] = NULL;
 		free(sadata->current_regdata);
 		return 1;
 	}
+
 	siril_log_color_message(_("Reference Image:\n"), "green");
 
 	if (regargs->matchSelection && regargs->selection.w > 0 && regargs->selection.h > 0) {
@@ -94,10 +101,11 @@ static int star_align_prepare_hook(struct generic_seq_args *args) {
 	else {
 		com.stars = peaker(&fit, regargs->layer, &com.starfinder_conf, &nb_stars, NULL, FALSE);
 	}
+
 	siril_log_message(_("Found %d stars in reference, channel #%d\n"), nb_stars, regargs->layer);
 
 
-	if (nb_stars < AT_MATCH_MINPAIRS) {
+	if (!com.stars || nb_stars < AT_MATCH_MINPAIRS) {
 		siril_log_message(
 				_("There are not enough stars in reference image to perform alignment\n"));
 		args->seq->regparam[regargs->layer] = NULL;
@@ -110,6 +118,7 @@ static int star_align_prepare_hook(struct generic_seq_args *args) {
 
 	sadata->ref.x = fit.rx;
 	sadata->ref.y = fit.ry;
+
 	clearfits(&fit);
 
 	if (regargs->x2upscale) {
@@ -131,8 +140,17 @@ static int star_align_prepare_hook(struct generic_seq_args *args) {
 	 */
 	i = 0;
 	sadata->refstars = malloc((MAX_STARS + 1) * sizeof(fitted_PSF *));
+	if (!sadata->refstars) {
+		PRINT_ALLOC_ERR;
+		return 1;
+	}
 	while (i < MAX_STARS && com.stars[i]) {
 		fitted_PSF *tmp = malloc(sizeof(fitted_PSF));
+		if (!tmp) {
+			PRINT_ALLOC_ERR;
+			sadata->refstars[i] = NULL;
+			return 1;
+		}
 		memcpy(tmp, com.stars[i], sizeof(fitted_PSF));
 		sadata->refstars[i] = tmp;
 		sadata->refstars[i+1] = NULL;
@@ -145,44 +163,63 @@ static int star_align_prepare_hook(struct generic_seq_args *args) {
 	} else {
 		sadata->fitted_stars = nb_stars;
 	}
-	FWHM_average(sadata->refstars, &FWHMx, &FWHMy, sadata->fitted_stars);
-	siril_log_message(_("FWHMx:%*.2f px\n"), 12, FWHMx);
-	siril_log_message(_("FWHMy:%*.2f px\n"), 12, FWHMy);
-	sadata->current_regdata[regargs->reference_image].fwhm = FWHMx;
+	FWHM_average(sadata->refstars, sadata->fitted_stars, &FWHMx, &FWHMy, &units);
+	siril_log_message(_("FWHMx:%*.2f %s\n"), 12, FWHMx, units);
+	siril_log_message(_("FWHMy:%*.2f %s\n"), 12, FWHMy, units);
 	sadata->current_regdata[regargs->reference_image].roundness = FWHMy/FWHMx;
+	sadata->current_regdata[regargs->reference_image].fwhm = FWHMx;
+	sadata->current_regdata[regargs->reference_image].weighted_fwhm = FWHMx;
 	
 	if (!regargs->translation_only) {
 		// allocate destination sequence data
 		regargs->imgparam = calloc(args->nb_filtered_images, sizeof(imgdata));
 		regargs->regparam = calloc(args->nb_filtered_images, sizeof(regdata));
-	}
-
-	if (args->seq->type == SEQ_SER) {
-		/* copied from ser_prepare_hook with one variation */
-		char dest[256];
-
-		args->new_ser = malloc(sizeof(struct ser_struct));
-
-		const char *ptr = strrchr(args->seq->seqname, G_DIR_SEPARATOR);
-		if (ptr)
-			snprintf(dest, 255, "%s%s.ser", regargs->prefix, ptr + 1);
-		else
-			snprintf(dest, 255, "%s%s.ser", regargs->prefix, args->seq->seqname);
-
-		/* Here the last argument is NULL because we do not want copy SER file
-		 * from the original. Indeed in the demosaicing case this would lead to
-		 * a wrong file (B&W and not in RAW data). Moreover, header informations
-		 * (like fps, local and UTC time, ...) have no sense now since some frames
-		 * could be removed from the sequence.
-		 */
-		if (ser_create_file(dest, args->new_ser, TRUE, NULL)) {
-			free(args->new_ser);
-			args->new_ser = NULL;
+		if (!regargs->imgparam  || !regargs->regparam) {
+			PRINT_ALLOC_ERR;
 			return 1;
 		}
-	}
+
+		if (args->seq->type == SEQ_SER) {
+			/* copied from seq_prepare_hook with one variation */
+			args->new_ser = malloc(sizeof(struct ser_struct));
+			if (!args->new_ser) {
+				PRINT_ALLOC_ERR;
+				return 1;
+			}
+
+			char dest[256];
+			const char *ptr = strrchr(args->seq->seqname, G_DIR_SEPARATOR);
+			if (ptr)
+				snprintf(dest, 255, "%s%s.ser", regargs->prefix, ptr + 1);
+			else
+				snprintf(dest, 255, "%s%s.ser", regargs->prefix, args->seq->seqname);
+
+			/* Here the last argument is NULL because we do not want copy SER file
+			 * from the original. Indeed in the demosaicing case this would lead to
+			 * a wrong file (B&W and not in RAW data). Moreover, header informations
+			 * (like fps, local and UTC time, ...) have no sense now since some frames
+			 * could be removed from the sequence.
+			 */
+			if (ser_create_file(dest, args->new_ser, TRUE, NULL)) {
+				free(args->new_ser);
+				args->new_ser = NULL;
+				return 1;
+			}
+
+			if (seq_prepare_writer(args))
+				return 1;
+		}
+		else if (args->seq->type == SEQ_FITSEQ) {
+			if (seq_prepare_hook(args))
+				return 1;
+		}
+}
 
 	sadata->success = calloc(args->nb_filtered_images, sizeof(BYTE));
+	if (!sadata->success) {
+		PRINT_ALLOC_ERR;
+		return 1;
+	}
 	return 0;
 }
 
@@ -192,11 +229,13 @@ static int star_align_prepare_hook(struct generic_seq_args *args) {
 static int star_align_image_hook(struct generic_seq_args *args, int out_index, int in_index, fits *fit, rectangle *_) {
 	struct star_align_data *sadata = args->user;
 	struct registration_args *regargs = sadata->regargs;
-	int nbpoints, i, nb_stars;
+	int nbpoints, nb_stars = 0;
+	int retvalue;
+	int nobj = 0;
+	int attempt = 1;
 	float FWHMx, FWHMy;
-	fitted_PSF **stars;
+	char *units;
 	Homography H = { 0 };
-	point image_size = { 0 };
 	int filenum = args->seq->imgparam[in_index].filenum;	// for display purposes
 
 	if (regargs->translation_only) {
@@ -207,20 +246,24 @@ static int star_align_image_hook(struct generic_seq_args *args, int out_index, i
 	}
 
 	if (in_index != regargs->reference_image) {
-		if (args->seq->type == SEQ_SER) {
+		fitted_PSF **stars;
+		if (args->seq->type == SEQ_SER || args->seq->type == SEQ_FITSEQ) {
 			siril_log_color_message(_("Frame %d:\n"), "bold", filenum);
 		}
+
 		if (regargs->matchSelection && regargs->selection.w > 0 && regargs->selection.h > 0) {
 			stars = peaker(fit, regargs->layer, &com.starfinder_conf, &nb_stars, &regargs->selection, FALSE);
 		}
 		else {
 			stars = peaker(fit, regargs->layer, &com.starfinder_conf, &nb_stars, NULL, FALSE);
 		}
-		siril_log_message(_("Found %d stars in image %d, channel #%d\n"), nb_stars, in_index, regargs->layer);
 
-		if (nb_stars < AT_MATCH_MINPAIRS) {
+		siril_log_message(_("Found %d stars in image %d, channel #%d\n"), nb_stars, filenum, regargs->layer);
+
+		if (!stars || nb_stars < AT_MATCH_MINPAIRS) {
 			siril_log_message(
 					_("Not enough stars. Image %d skipped\n"), filenum);
+			if (stars) free_fitted_stars(stars);
 			return 1;
 		}
 
@@ -233,43 +276,68 @@ static int star_align_image_hook(struct generic_seq_args *args, int out_index, i
 		else {
 			nbpoints = nb_stars;
 		}
-		image_size.x = fit->rx;
-		image_size.y = fit->ry;
-		if (new_star_match(stars, sadata->refstars, nbpoints, &H, image_size)) {
-			siril_log_color_message(_("Cannot perform star matching. Image %d skipped\n"),
-					"red", filenum);
-			i = 0;
-			while (i < MAX_STARS && stars[i])
-				free(stars[i++]);
-			free(stars);
+
+		/* make a loop with different tries in order to align the two sets of data */
+		double scale_min = 0.9;
+		double scale_max = 1.1;
+		retvalue = 1;
+		while (retvalue && attempt < NB_OF_MATCHING_TRY){
+			retvalue = new_star_match(stars, sadata->refstars, nbpoints, nobj, scale_min, scale_max, &H, FALSE);
+			if (attempt == 1) {
+				scale_min = -1.0;
+				scale_max = -1.0;
+			} else {
+				nobj += 50;
+			}
+			attempt++;
+		}
+		if (retvalue) {
+			siril_log_color_message(_("Cannot perform star matching: try #%d. Image %d skipped\n"),
+					"red", attempt, filenum);
+			free_fitted_stars(stars);
+			return 1;
+		}
+		if (H.pair_matched < regargs->min_pairs) {
+			siril_log_color_message(_("Not enough star pairs (%d): Image %d skipped\n"),
+					"red", H.pair_matched, filenum);
+			free_fitted_stars(stars);
 			return 1;
 		}
 
-		FWHM_average(stars, &FWHMx, &FWHMy, nbpoints);
+		FWHM_average(stars, nbpoints, &FWHMx, &FWHMy, &units);
+#ifdef _OPENMP
 #pragma omp critical
-		print_alignment_results(H, filenum, FWHMx, FWHMy);
-		sadata->current_regdata[in_index].fwhm = FWHMx;
+#endif
+		print_alignment_results(H, filenum, FWHMx, FWHMy, units);
+
 		sadata->current_regdata[in_index].roundness = FWHMy/FWHMx;
+		sadata->current_regdata[in_index].fwhm =  FWHMx;
+		sadata->current_regdata[in_index].weighted_fwhm = 2 * FWHMx
+				* (((double) sadata->fitted_stars) - (double) H.pair_matched)
+				/ (double) sadata->fitted_stars + FWHMx;
 
 		if (!regargs->translation_only) {
 			if (regargs->x2upscale) {
-				double upscale = 2.0;
-
-				cvResizeGaussian(fit, fit->rx * upscale, fit->ry * upscale, OPENCV_NEAREST);
-
-				cvApplyScaleToH(&H, upscale);
+				if (cvResizeGaussian(fit, fit->rx * 2, fit->ry * 2, OPENCV_NEAREST)) {
+					free_fitted_stars(stars);
+					return 1;
+				}
+				cvApplyScaleToH(&H, 2.0);
 			}
-			cvTransformImage(fit, sadata->ref, H, regargs->interpolation);
+			fits_flip_top_to_bottom(fit);
+			if (cvTransformImage(fit, (long) sadata->ref.x, (long) sadata->ref.y, H, regargs->interpolation)) {
+				free_fitted_stars(stars);
+				return 1;
+			}
+			fits_flip_top_to_bottom(fit);
 		}
 
-		i = 0;
-		while (i < MAX_STARS && stars[i])
-			free(stars[i++]);
-		free(stars);
+		free_fitted_stars(stars);
 	}
 	else {
 		if (regargs->x2upscale && !regargs->translation_only) {
-			cvResizeGaussian(fit, fit->rx * 2, fit->ry * 2, OPENCV_NEAREST);
+			if (cvResizeGaussian(fit, fit->rx * 2, fit->ry * 2, OPENCV_NEAREST))
+				return 1;
 		}
 	}
 
@@ -277,10 +345,12 @@ static int star_align_image_hook(struct generic_seq_args *args, int out_index, i
 		regargs->imgparam[out_index].filenum = args->seq->imgparam[in_index].filenum;
 		regargs->imgparam[out_index].incl = SEQUENCE_DEFAULT_INCLUDE;
 		regargs->regparam[out_index].fwhm = sadata->current_regdata[in_index].fwhm;	// not FWHMx because of the ref frame
+		regargs->regparam[out_index].weighted_fwhm = sadata->current_regdata[in_index].weighted_fwhm;
 		regargs->regparam[out_index].roundness = sadata->current_regdata[in_index].roundness;
+		invalidate_stats_from_fit(fit);
 	} else {
-		sadata->current_regdata[in_index].shiftx = +H.h02;
-		sadata->current_regdata[in_index].shifty = -H.h12;
+		set_shifts(args->seq, in_index, regargs->layer, (float) H.h02,
+				(float) -H.h12, fit->top_down);
 		args->seq->imgparam[out_index].incl = SEQUENCE_DEFAULT_INCLUDE;
 	}
 	sadata->success[out_index] = 1;
@@ -290,12 +360,12 @@ static int star_align_image_hook(struct generic_seq_args *args, int out_index, i
 static int star_align_finalize_hook(struct generic_seq_args *args) {
 	struct star_align_data *sadata = args->user;
 	struct registration_args *regargs = sadata->regargs;
-	int i = 0, failed = 0;
-	if (sadata->refstars) {
-		while (i < MAX_STARS && sadata->refstars[i])
-			free(sadata->refstars[i++]);
-		free(sadata->refstars);
-	}
+	int i, failed = 0;
+
+	// images may have been excluded but selnum wasn't updated
+	fix_selnum(args->seq, FALSE);
+
+	free_fitted_stars(sadata->refstars);
 
 	if (!args->retval) {
 		for (i = 0; i < args->nb_filtered_images; i++)
@@ -303,57 +373,62 @@ static int star_align_finalize_hook(struct generic_seq_args *args) {
 				failed++;
 		regargs->new_total = args->nb_filtered_images - failed;
 
-		// regargs->imgparam and regargs->regparam may have holes caused by images
-		// that failed to be registered
-		if (failed && !regargs->translation_only) {
-			int j;
-
-			for (i = 0, j = 0; i < regargs->new_total; i++, j++) {
-				while (!sadata->success[j] && j < args->nb_filtered_images) j++;
-				assert(sadata->success[j]);
-				if (i != j) {
-					regargs->imgparam[i] = regargs->imgparam[j];
-					regargs->regparam[i] = regargs->regparam[j];
+		if (!regargs->translation_only) {
+			if (failed) {
+				// regargs->imgparam and regargs->regparam may have holes caused by images
+				// that failed to be registered - compact them
+				int j;
+				for (i = 0, j = 0; i < regargs->new_total; i++, j++) {
+					while (!sadata->success[j] && j < args->nb_filtered_images) j++;
+					g_assert(sadata->success[j]);
+					if (i != j) {
+						regargs->imgparam[i] = regargs->imgparam[j];
+						regargs->regparam[i] = regargs->regparam[j];
+					}
 				}
 			}
-			// compact the sequence: there will be holes for images
-			// that failed to be registered if we don't do that
-			if (args->seq->type == SEQ_SER)
-				ser_compact_file(args->new_ser, sadata->success, args->nb_filtered_images);
-		}
 
-		// same as ser_finalize_hook()
-		if (args->seq->type == SEQ_SER) {
-			ser_write_and_close(args->new_ser);
-			free(args->new_ser);
+			seq_finalize_hook(args);
 		}
-
-		args->seq->regparam[regargs->layer] = sadata->current_regdata;
 	} else {
 		regargs->new_total = 0;
+		free(args->seq->regparam[regargs->layer]);
+		args->seq->regparam[regargs->layer] = NULL;
+
+		// args->new_ser can be null if stars were not detected in the reference image
+		// same as seq_finalize_hook but with file deletion
+		if ((args->force_ser_output || args->seq->type == SEQ_SER) && args->new_ser) {
+			ser_close_and_delete_file(args->new_ser);
+			free(args->new_ser);
+		}
+		else if ((args->force_fitseq_output || args->seq->type == SEQ_FITSEQ) && args->new_fitseq) {
+			fitseq_close_and_delete_file(args->new_fitseq);
+			free(args->new_fitseq);
+		}
 	}
+
 	if (sadata->success) free(sadata->success);
 	free(sadata);
 	args->user = NULL;
-
-	update_used_memory();
+	clear_stars_list();
 
 	if (!args->retval) {
 		siril_log_message(_("Registration finished.\n"));
-		siril_log_color_message(_("%d images processed.\n"), "green",
-				args->nb_filtered_images);
-		siril_log_color_message(_("Total: %d failed, %d registered.\n"), "green",
-				failed, regargs->new_total);
+		gchar *str = ngettext("%d image processed.\n", "%d images processed.\n", args->nb_filtered_images);
+		str = g_strdup_printf(str, args->nb_filtered_images);
+		siril_log_color_message(str, "green");
+		siril_log_color_message(_("Total: %d failed, %d registered.\n"), "green", failed, regargs->new_total);
+
+		g_free(str);
 		if (!regargs->translation_only) {
 			// explicit sequence creation to copy imgparam and regparam
 			create_output_sequence_for_global_star(regargs);
 			// will be loaded in the idle function if (load_new_sequence)
+			regargs->load_new_sequence = TRUE; // only case where a new sequence must be loaded
 		}
-		else regargs->load_new_sequence = FALSE;
 	}
 	else {
 		siril_log_message(_("Registration aborted.\n"));
-		regargs->load_new_sequence = FALSE;
 	}
 	return regargs->new_total == 0;
 	// TODO: args is never freed because we don't call an end function for
@@ -363,29 +438,71 @@ static int star_align_finalize_hook(struct generic_seq_args *args) {
 }
 
 int register_star_alignment(struct registration_args *regargs) {
-	struct generic_seq_args *args = malloc(sizeof(struct generic_seq_args));
-	args->seq = regargs->seq;
-	args->partial_image = FALSE;
-	if (regargs->process_all_frames) {
-		args->filtering_criterion = seq_filter_all;
-		args->nb_filtered_images = regargs->seq->number;
-	} else {
+	struct generic_seq_args *args = create_default_seqargs(regargs->seq);
+	if (!regargs->process_all_frames) {
 		args->filtering_criterion = seq_filter_included;
 		args->nb_filtered_images = regargs->seq->selnum;
 	}
 	args->prepare_hook = star_align_prepare_hook;
 	args->image_hook = star_align_image_hook;
-	args->save_hook = NULL;
 	args->finalize_hook = star_align_finalize_hook;
-	args->idle_function = NULL;
 	args->stop_on_error = FALSE;
 	args->description = _("Global star registration");
 	args->has_output = !regargs->translation_only;
+	args->output_type = get_data_type(args->seq->bitpix);
+	args->upscale_ratio = regargs->x2upscale ? 2.0 : 1.0;
 	args->new_seq_prefix = regargs->prefix;
 	args->load_new_sequence = TRUE;
-	args->force_ser_output = FALSE;
 	args->already_in_a_thread = TRUE;
-	args->parallel = TRUE;
+	unsigned int MB_per_image; int MB_avail;
+	int nb_images = compute_nb_images_fit_memory(regargs->seq, args->upscale_ratio, FALSE, &MB_per_image, &MB_avail);
+	if (nb_images > 0) {
+		/* The registration memory consumption, n is image size and m channel size.
+		 * First, a threshold is computed for star pixel value, using statistics:
+		 *	ushort data: O(m), data is duplicated for median
+		 *		computation if there are nil values, O(1) otherwise
+		 *	float data: O(1)
+		 * Then, still in peaker(), image is filtered using wavelets, duplicating the
+		 * image channel to act as input and output of the filter (O(m)) and
+		 * wavelet_transform then allocates 3 times the size of the channel in float,
+		 * because we request 3 planes of wavelets, and pave_2d allocates it once more
+		 * for its working copy, so that makes O(5m as float).
+		 * Then, in the image is written by the generic function if rotation is enabled:
+		 * cvTransformImage is O(n) in mem for monochrome and O(2n) for color.
+		 * Since these three operations are in sequence, we need room only for the
+		 * biggest. In case of float color image, it's the rotation that takes more
+		 * memory, with O(2n) = O(6m). In other cases it's the wavelets with O(5m) or
+		 * O(10m) when it's ushort.
+		 * Don't forget that this is in addition to the image being already read.
+		 */
+		unsigned int required;
+		if (args->has_output && regargs->seq->nb_layers == 3 &&
+				get_data_type(regargs->seq->bitpix) == DATA_FLOAT)
+			required = MB_per_image * 3;
+		else {
+			unsigned int MB_per_channel = regargs->seq->nb_layers == 1 ? MB_per_image : MB_per_image / 3;
+			unsigned int float_multiplier = get_data_type(regargs->seq->bitpix) == DATA_FLOAT ? 1 : 2;
+			required = MB_per_image + 5 * MB_per_channel * float_multiplier;
+		}
+		args->max_thread = MB_avail / required;
+		siril_debug_print("Memory required per thread: %u MB, limiting to %d threads\n", required, args->max_thread);
+	}
+
+	if (args->max_thread > com.max_thread || args->max_thread < 0)
+		args->max_thread = com.max_thread;
+	if (args->max_thread == 0) {
+		gchar *mem_per_image = g_format_size_full(MB_per_image * BYTES_IN_A_MB, G_FORMAT_SIZE_IEC_UNITS);
+		gchar *mem_available = g_format_size_full(MB_avail * BYTES_IN_A_MB, G_FORMAT_SIZE_IEC_UNITS);
+
+		siril_log_color_message(_("Not enough memory to do this operation (%s required per image, %s considered available)\n"),
+				"red", mem_per_image, mem_available);
+
+		g_free(mem_per_image);
+		g_free(mem_available);
+		free(args);
+		return -1;
+	}
+	siril_log_message(_("With the current memory and thread limits, up to %d thread(s) can be used for registration\n"), args->max_thread);
 
 	struct star_align_data *sadata = calloc(1, sizeof(struct star_align_data));
 	if (!sadata) {
@@ -396,7 +513,9 @@ int register_star_alignment(struct registration_args *regargs) {
 	args->user = sadata;
 
 	generic_sequence_worker(args);
-	return args->retval;
+	regargs->retval = args->retval;
+	free(args);
+	return regargs->retval;
 }
 
 static void create_output_sequence_for_global_star(struct registration_args *args) {
@@ -434,14 +553,17 @@ static void create_output_sequence_for_global_star(struct registration_args *arg
 	free_sequence(&seq, FALSE);
 }
 
-static void print_alignment_results(Homography H, int filenum, float FWHMx, float FWHMy) {
+static void print_alignment_results(Homography H, int filenum, float FWHMx, float FWHMy, char *units) {
 	double rotation, scale, scaleX, scaleY;
 	point shift;
 	double inliers;
 
 	/* Matching information */
 	siril_log_color_message(_("Matching stars in image %d: done\n"), "green", filenum);
-	siril_log_message(_("%d pair matches.\n"), H.pair_matched);
+	gchar *str = ngettext("%d pair match.\n", "%d pair matches.\n", H.pair_matched);
+	str = g_strdup_printf(str, H.pair_matched);
+	siril_log_message(str);
+	g_free(str);
 	inliers = 1.0 - ((((double) H.pair_matched - (double) H.Inliers)) / (double) H.pair_matched);
 	siril_log_message(_("Inliers:%*.3f\n"), 11, inliers);
 
@@ -454,15 +576,15 @@ static void print_alignment_results(Homography H, int filenum, float FWHMx, floa
 	siril_log_message(_("scale:%*.3f\n"), 13, scale);
 
 	/* Rotation */
-	rotation = -atan2(H.h01, H.h00);
-	siril_log_message(_("rotation:%+*.2f deg\n"), 9, rotation * 180 / M_PI);
+	rotation = atan2(H.h01, H.h00) * 180 / M_PI;
+	siril_log_message(_("rotation:%+*.3f deg\n"), 9, rotation);
 
 	/* Translation */
 	shift.x = -H.h02;
 	shift.y = -H.h12;
 	siril_log_message(_("dx:%+*.2f px\n"), 15, shift.x);
 	siril_log_message(_("dy:%+*.2f px\n"), 15, shift.y);
-	siril_log_message(_("FWHMx:%*.2f px\n"), 12, FWHMx);
-	siril_log_message(_("FWHMy:%*.2f px\n"), 12, FWHMy);
+	siril_log_message(_("FWHMx:%*.2f %s\n"), 12, FWHMx, units);
+	siril_log_message(_("FWHMy:%*.2f %s\n"), 12, FWHMy, units);
 }
 
